# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import Location


class SearchChatsNearby(BaseObject):
    """
    Returns a list of users and location-based supergroups nearby. The list of users nearby will be updated for 60 seconds after the request by the updates updateUsersNearby. The request should be sent again every 25 seconds with adjusted location to not miss new chats
    
    Params:
        location (:class:`Location`)
            Current user location
        
    """

    ID: str = Field("searchChatsNearby", alias="@type")
    location: Location

    @staticmethod
    def read(q: dict) -> SearchChatsNearby:
        return SearchChatsNearby.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .chat_statistics_administrator_actions_info import ChatStatisticsAdministratorActionsInfo
from .chat_statistics_inviter_info import ChatStatisticsInviterInfo
from .chat_statistics_message_interaction_info import ChatStatisticsMessageInteractionInfo
from .chat_statistics_message_sender_info import ChatStatisticsMessageSenderInfo
from .date_range import DateRange
from .statistical_graph import StatisticalGraph
from .statistical_value import StatisticalValue
from ..base_object import BaseObject


class ChatStatistics(BaseObject):
    """
    Contains a detailed statistics about a chat
    
    """

    ID: str = Field("chatStatistics", alias="@type")


class ChatStatisticsChannel(ChatStatistics):
    """
    A detailed statistics about a channel chat
    
    Params:
        period (:class:`DateRange`)
            A period to which the statistics applies
        
        member_count (:class:`StatisticalValue`)
            Number of members in the chat
        
        mean_view_count (:class:`StatisticalValue`)
            Mean number of times the recently sent messages was viewed
        
        mean_share_count (:class:`StatisticalValue`)
            Mean number of times the recently sent messages was shared
        
        enabled_notifications_percentage (:class:`float`)
            A percentage of users with enabled notifications for the chat
        
        member_count_graph (:class:`StatisticalGraph`)
            A graph containing number of members in the chat
        
        join_graph (:class:`StatisticalGraph`)
            A graph containing number of members joined and left the chat
        
        mute_graph (:class:`StatisticalGraph`)
            A graph containing number of members muted and unmuted the chat
        
        view_count_by_hour_graph (:class:`StatisticalGraph`)
            A graph containing number of message views in a given hour in the last two weeks
        
        view_count_by_source_graph (:class:`StatisticalGraph`)
            A graph containing number of message views per source
        
        join_by_source_graph (:class:`StatisticalGraph`)
            A graph containing number of new member joins per source
        
        language_graph (:class:`StatisticalGraph`)
            A graph containing number of users viewed chat messages per language
        
        message_interaction_graph (:class:`StatisticalGraph`)
            A graph containing number of chat message views and shares
        
        instant_view_interaction_graph (:class:`StatisticalGraph`)
            A graph containing number of views of associated with the chat instant views
        
        recent_message_interactions (:obj:`list[ChatStatisticsMessageInteractionInfo]`)
            Detailed statistics about number of views and shares of recently sent messages
        
    """

    ID: str = Field("chatStatisticsChannel", alias="@type")
    period: DateRange
    member_count: StatisticalValue
    mean_view_count: StatisticalValue
    mean_share_count: StatisticalValue
    enabled_notifications_percentage: float
    member_count_graph: StatisticalGraph
    join_graph: StatisticalGraph
    mute_graph: StatisticalGraph
    view_count_by_hour_graph: StatisticalGraph
    view_count_by_source_graph: StatisticalGraph
    join_by_source_graph: StatisticalGraph
    language_graph: StatisticalGraph
    message_interaction_graph: StatisticalGraph
    instant_view_interaction_graph: StatisticalGraph
    recent_message_interactions: list[ChatStatisticsMessageInteractionInfo]

    @staticmethod
    def read(q: dict) -> ChatStatisticsChannel:
        return ChatStatisticsChannel.construct(**q)


class ChatStatisticsSupergroup(ChatStatistics):
    """
    A detailed statistics about a supergroup chat
    
    Params:
        period (:class:`DateRange`)
            A period to which the statistics applies
        
        member_count (:class:`StatisticalValue`)
            Number of members in the chat
        
        message_count (:class:`StatisticalValue`)
            Number of messages sent to the chat
        
        viewer_count (:class:`StatisticalValue`)
            Number of users who viewed messages in the chat
        
        sender_count (:class:`StatisticalValue`)
            Number of users who sent messages to the chat
        
        member_count_graph (:class:`StatisticalGraph`)
            A graph containing number of members in the chat
        
        join_graph (:class:`StatisticalGraph`)
            A graph containing number of members joined and left the chat
        
        join_by_source_graph (:class:`StatisticalGraph`)
            A graph containing number of new member joins per source
        
        language_graph (:class:`StatisticalGraph`)
            A graph containing distribution of active users per language
        
        message_content_graph (:class:`StatisticalGraph`)
            A graph containing distribution of sent messages by content type
        
        action_graph (:class:`StatisticalGraph`)
            A graph containing number of different actions in the chat
        
        day_graph (:class:`StatisticalGraph`)
            A graph containing distribution of message views per hour
        
        week_graph (:class:`StatisticalGraph`)
            A graph containing distribution of message views per day of week
        
        top_senders (:obj:`list[ChatStatisticsMessageSenderInfo]`)
            List of users sent most messages in the last week
        
        top_administrators (:obj:`list[ChatStatisticsAdministratorActionsInfo]`)
            List of most active administrators in the last week
        
        top_inviters (:obj:`list[ChatStatisticsInviterInfo]`)
            List of most active inviters of new members in the last week
        
    """

    ID: str = Field("chatStatisticsSupergroup", alias="@type")
    period: DateRange
    member_count: StatisticalValue
    message_count: StatisticalValue
    viewer_count: StatisticalValue
    sender_count: StatisticalValue
    member_count_graph: StatisticalGraph
    join_graph: StatisticalGraph
    join_by_source_graph: StatisticalGraph
    language_graph: StatisticalGraph
    message_content_graph: StatisticalGraph
    action_graph: StatisticalGraph
    day_graph: StatisticalGraph
    week_graph: StatisticalGraph
    top_senders: list[ChatStatisticsMessageSenderInfo]
    top_administrators: list[ChatStatisticsAdministratorActionsInfo]
    top_inviters: list[ChatStatisticsInviterInfo]

    @staticmethod
    def read(q: dict) -> ChatStatisticsSupergroup:
        return ChatStatisticsSupergroup.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class SetDatabaseEncryptionKey(BaseObject):
    """
    Changes the database encryption key. Usually the encryption key is never changed and is stored in some OS keychain
    
    Params:
        new_encryption_key (:class:`str`)
            New encryption key
        
    """

    ID: str = Field("setDatabaseEncryptionKey", alias="@type")
    new_encryption_key: str

    @staticmethod
    def read(q: dict) -> SetDatabaseEncryptionKey:
        return SetDatabaseEncryptionKey.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class Destroy(BaseObject):
    """
    Closes the TDLib instance, destroying all local data without a proper logout. The current user session will remain in the list of all active sessions. All local data will be destroyed. After the destruction completes updateAuthorizationState with authorizationStateClosed will be sent. Can be called before authorization
    
    """

    ID: str = Field("destroy", alias="@type")

    @staticmethod
    def read(q: dict) -> Destroy:
        return Destroy.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import PublicChatType


class CheckCreatedPublicChatsLimit(BaseObject):
    """
    Checks whether the maximum number of owned public chats has been reached. Returns corresponding error if the limit was reached
    
    Params:
        type_ (:class:`PublicChatType`)
            Type of the public chats, for which to check the limit
        
    """

    ID: str = Field("checkCreatedPublicChatsLimit", alias="@type")
    type_: PublicChatType = Field(..., alias='type')

    @staticmethod
    def read(q: dict) -> CheckCreatedPublicChatsLimit:
        return CheckCreatedPublicChatsLimit.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetMessageImportConfirmationText(BaseObject):
    """
    Returns a confirmation text to be shown to the user before starting message import
    
    Params:
        chat_id (:class:`int`)
            Identifier of a chat to which the messages will be imported. It must be an identifier of a private chat with a mutual contact or an identifier of a supergroup chat with can_change_info administrator right
        
    """

    ID: str = Field("getMessageImportConfirmationText", alias="@type")
    chat_id: int

    @staticmethod
    def read(q: dict) -> GetMessageImportConfirmationText:
        return GetMessageImportConfirmationText.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .animation import Animation
from .audio import Audio
from .contact import Contact
from .document import Document
from .game import Game
from .location import Location
from .photo import Photo
from .sticker import Sticker
from .thumbnail import Thumbnail
from .venue import Venue
from .video import Video
from .voice_note import VoiceNote
from ..base_object import BaseObject


class InlineQueryResult(BaseObject):
    """
    Represents a single result of an inline query
    
    """

    ID: str = Field("inlineQueryResult", alias="@type")


class InlineQueryResultAnimation(InlineQueryResult):
    """
    Represents an animation file
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        animation (:class:`Animation`)
            Animation file
        
        title (:class:`str`)
            Animation title
        
    """

    ID: str = Field("inlineQueryResultAnimation", alias="@type")
    id: str
    animation: Animation
    title: str

    @staticmethod
    def read(q: dict) -> InlineQueryResultAnimation:
        return InlineQueryResultAnimation.construct(**q)


class InlineQueryResultArticle(InlineQueryResult):
    """
    Represents a link to an article or web page
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        url (:class:`str`)
            URL of the result, if it exists
        
        hide_url (:class:`bool`)
            True, if the URL must be not shown
        
        title (:class:`str`)
            Title of the result
        
        param_description (:class:`str`)
            A short description of the result
        
        thumbnail (:class:`Thumbnail`)
            Result thumbnail in JPEG format; may be null
        
    """

    ID: str = Field("inlineQueryResultArticle", alias="@type")
    id: str
    url: str
    hide_url: bool
    title: str
    param_description: str
    thumbnail: typing.Optional[Thumbnail] = None

    @staticmethod
    def read(q: dict) -> InlineQueryResultArticle:
        return InlineQueryResultArticle.construct(**q)


class InlineQueryResultAudio(InlineQueryResult):
    """
    Represents an audio file
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        audio (:class:`Audio`)
            Audio file
        
    """

    ID: str = Field("inlineQueryResultAudio", alias="@type")
    id: str
    audio: Audio

    @staticmethod
    def read(q: dict) -> InlineQueryResultAudio:
        return InlineQueryResultAudio.construct(**q)


class InlineQueryResultContact(InlineQueryResult):
    """
    Represents a user contact
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        contact (:class:`Contact`)
            A user contact
        
        thumbnail (:class:`Thumbnail`)
            Result thumbnail in JPEG format; may be null
        
    """

    ID: str = Field("inlineQueryResultContact", alias="@type")
    id: str
    contact: Contact
    thumbnail: typing.Optional[Thumbnail] = None

    @staticmethod
    def read(q: dict) -> InlineQueryResultContact:
        return InlineQueryResultContact.construct(**q)


class InlineQueryResultDocument(InlineQueryResult):
    """
    Represents a document
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        document (:class:`Document`)
            Document
        
        title (:class:`str`)
            Document title
        
        param_description (:class:`str`)
            Document description
        
    """

    ID: str = Field("inlineQueryResultDocument", alias="@type")
    id: str
    document: Document
    title: str
    param_description: str

    @staticmethod
    def read(q: dict) -> InlineQueryResultDocument:
        return InlineQueryResultDocument.construct(**q)


class InlineQueryResultGame(InlineQueryResult):
    """
    Represents information about a game
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        game (:class:`Game`)
            Game result
        
    """

    ID: str = Field("inlineQueryResultGame", alias="@type")
    id: str
    game: Game

    @staticmethod
    def read(q: dict) -> InlineQueryResultGame:
        return InlineQueryResultGame.construct(**q)


class InlineQueryResultLocation(InlineQueryResult):
    """
    Represents a point on the map
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        location (:class:`Location`)
            Location result
        
        title (:class:`str`)
            Title of the result
        
        thumbnail (:class:`Thumbnail`)
            Result thumbnail in JPEG format; may be null
        
    """

    ID: str = Field("inlineQueryResultLocation", alias="@type")
    id: str
    location: Location
    title: str
    thumbnail: typing.Optional[Thumbnail] = None

    @staticmethod
    def read(q: dict) -> InlineQueryResultLocation:
        return InlineQueryResultLocation.construct(**q)


class InlineQueryResultPhoto(InlineQueryResult):
    """
    Represents a photo
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        photo (:class:`Photo`)
            Photo
        
        title (:class:`str`)
            Title of the result, if known
        
        param_description (:class:`str`)
            A short description of the result, if known
        
    """

    ID: str = Field("inlineQueryResultPhoto", alias="@type")
    id: str
    photo: Photo
    title: str
    param_description: str

    @staticmethod
    def read(q: dict) -> InlineQueryResultPhoto:
        return InlineQueryResultPhoto.construct(**q)


class InlineQueryResultSticker(InlineQueryResult):
    """
    Represents a sticker
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        sticker (:class:`Sticker`)
            Sticker
        
    """

    ID: str = Field("inlineQueryResultSticker", alias="@type")
    id: str
    sticker: Sticker

    @staticmethod
    def read(q: dict) -> InlineQueryResultSticker:
        return InlineQueryResultSticker.construct(**q)


class InlineQueryResultVenue(InlineQueryResult):
    """
    Represents information about a venue
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        venue (:class:`Venue`)
            Venue result
        
        thumbnail (:class:`Thumbnail`)
            Result thumbnail in JPEG format; may be null
        
    """

    ID: str = Field("inlineQueryResultVenue", alias="@type")
    id: str
    venue: Venue
    thumbnail: typing.Optional[Thumbnail] = None

    @staticmethod
    def read(q: dict) -> InlineQueryResultVenue:
        return InlineQueryResultVenue.construct(**q)


class InlineQueryResultVideo(InlineQueryResult):
    """
    Represents a video
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        video (:class:`Video`)
            Video
        
        title (:class:`str`)
            Title of the video
        
        param_description (:class:`str`)
            Description of the video
        
    """

    ID: str = Field("inlineQueryResultVideo", alias="@type")
    id: str
    video: Video
    title: str
    param_description: str

    @staticmethod
    def read(q: dict) -> InlineQueryResultVideo:
        return InlineQueryResultVideo.construct(**q)


class InlineQueryResultVoiceNote(InlineQueryResult):
    """
    Represents a voice note
    
    Params:
        id (:class:`str`)
            Unique identifier of the query result
        
        voice_note (:class:`VoiceNote`)
            Voice note
        
        title (:class:`str`)
            Title of the voice note
        
    """

    ID: str = Field("inlineQueryResultVoiceNote", alias="@type")
    id: str
    voice_note: VoiceNote
    title: str

    @staticmethod
    def read(q: dict) -> InlineQueryResultVoiceNote:
        return InlineQueryResultVoiceNote.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetFileExtension(BaseObject):
    """
    Returns the extension of a file, guessed by its MIME type. Returns an empty string on failure. Can be called synchronously
    
    Params:
        mime_type (:class:`str`)
            The MIME type of the file
        
    """

    ID: str = Field("getFileExtension", alias="@type")
    mime_type: str

    @staticmethod
    def read(q: dict) -> GetFileExtension:
        return GetFileExtension.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class SearchPublicChats(BaseObject):
    """
    Searches public chats by looking for specified query in their username and title. Currently only private chats, supergroups and channels can be public. Returns a meaningful number of results. Returns nothing if the length of the searched username prefix is less than 5. Excludes private chats with contacts and chats from the chat list from the results
    
    Params:
        query (:class:`str`)
            Query to search for
        
    """

    ID: str = Field("searchPublicChats", alias="@type")
    query: str

    @staticmethod
    def read(q: dict) -> SearchPublicChats:
        return SearchPublicChats.construct(**q)



import re
from abc import (
    ABC,
    abstractmethod,
)
from collections import (
    Callable,
    Coroutine,
)
from typing import (
    Any,
    NoReturn,
    Optional,
    Union,
)

from aiotdlib.api import (
    BaseObject,
    MessageAnimation,
    MessageAudio,
    MessageContact,
    MessageDice,
    MessageDocument,
    MessageGame,
    MessageInvoice,
    MessageLocation,
    MessagePhoto,
    MessagePoll,
    MessageSticker,
    MessageText,
    MessageUnsupported,
    MessageVenue,
    MessageVideo,
    MessageVideoNote,
    MessageVoiceNote,
    UpdateNewMessage,
)

FilterCallable = Callable[[BaseObject], Coroutine[Any, Any, bool]]


class BaseFilter(ABC):
    _name = None

    @abstractmethod
    async def __call__(self, update: BaseObject) -> Optional[Union[bool, dict]]:
        return True

    def __and__(self, other: 'BaseFilter') -> 'BaseFilter':
        return MergedFilter(self, and_filter=other)

    def __or__(self, other: 'BaseFilter') -> 'BaseFilter':
        return MergedFilter(self, or_filter=other)

    def __xor__(self, other: 'BaseFilter') -> 'BaseFilter':
        return XORFilter(self, other)

    def __invert__(self) -> 'BaseFilter':
        return InvertedFilter(self)

    def __repr__(self) -> str:
        # We do this here instead of in a __init__ so filter don't have to call __init__ or super()
        if self.name is None:
            self.name = self.__class__.__name__

        return self.name

    @property
    def name(self) -> Optional[str]:
        return self._name

    @name.setter
    def name(self, name: Optional[str]) -> None:
        self._name = name


class BaseObjectFilter(BaseFilter, ABC):
    data_filter = False  # If True - dict, returned from filter() will be merged into update.EXTRA

    async def __call__(self, update: BaseObject) -> Optional[Union[bool, dict]]:
        super_result = await super().__call__(update)

        if not bool(super_result):
            return False

        return await self.filter(update)

    @abstractmethod
    async def filter(self, update: BaseObject) -> Optional[Union[bool, dict]]:
        pass


class InvertedFilter(BaseObjectFilter):
    def __init__(self, f: BaseFilter):
        self.f = f

    async def filter(self, update: BaseObject) -> bool:
        result = await self.f(update)
        return not bool(result)

    @property
    def name(self) -> str:
        return f"<inverted {self.f}>"

    @name.setter
    def name(self, name: str) -> NoReturn:
        raise RuntimeError('Cannot set name for InvertedFilter')


class MergedFilter(BaseObjectFilter):
    """Represents a filter consisting of two other filters.

    Args:
        base_filter: Filter 1 of the merged filter.
        and_filter: Optional filter to "and" with base_filter. Mutually exclusive with or_filter.
        or_filter: Optional filter to "or" with base_filter. Mutually exclusive with and_filter.

    """

    def __init__(
            self,
            base_filter: BaseFilter,
            and_filter: BaseFilter = None,
            or_filter: BaseFilter = None
    ):
        self.base_filter = base_filter

        if self.base_filter.data_filter:
            self.data_filter = True

        self.and_filter = and_filter

        if self.and_filter and not isinstance(self.and_filter, bool) and self.and_filter.data_filter:
            self.data_filter = True

        self.or_filter = or_filter

        if self.or_filter and not isinstance(self.and_filter, bool) and self.or_filter.data_filter:
            self.data_filter = True

    async def filter(self, update: BaseObject) -> Union[bool, dict]:  # pylint: disable=R0911
        base_output = await self.base_filter(update)

        # We need to check if the filters are data filters and if so return the merged data.
        # If it's not a data filter or an or_filter but no matches return bool
        if self.and_filter:
            # And filter needs to short circuit if base is falsey
            if base_output:
                comp_output = await self.and_filter(update)

                if comp_output:
                    if self.data_filter:
                        merged = (base_output or {}) | (comp_output or {})

                        if merged:
                            return merged

                    return True
        elif self.or_filter:
            # Or filter needs to short circuit if base is truthey
            if base_output:
                if self.data_filter:
                    return base_output

                return True

            comp_output = await self.or_filter(update)

            if comp_output:
                if self.data_filter:
                    return comp_output

                return True

        return False

    @property
    def name(self) -> str:
        return (
            f"<{self.base_filter} {'and' if self.and_filter else 'or'} "
            f"{self.and_filter or self.or_filter}>"
        )

    @name.setter
    def name(self, name: str) -> NoReturn:
        raise RuntimeError('Cannot set name for MergedFilter')


class XORFilter(BaseObjectFilter):
    """
    Convenience filter acting as wrapper for :class:`MergedFilter` representing the an XOR gate
    for two filters.

    Args:
        base_filter: Filter 1 of the merged filter.
        xor_filter: Filter 2 of the merged filter.

    """

    def __init__(self, base_filter: BaseFilter, xor_filter: BaseFilter):
        self.base_filter = base_filter
        self.xor_filter = xor_filter
        self.merged_filter = (base_filter & ~xor_filter) | (~base_filter & xor_filter)

    async def filter(self, update: BaseObject) -> Optional[Union[bool, dict]]:
        return await self.merged_filter(update)

    @property
    def name(self) -> str:
        return f'<{self.base_filter} xor {self.xor_filter}>'

    @name.setter
    def name(self, name: str) -> NoReturn:
        raise RuntimeError('Cannot set name for XORFilter')


# Predefined filters
class AllFilter(BaseObjectFilter):
    name = 'Filters.all'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return True


class MessageFilter(BaseObjectFilter):
    name = 'Filters.message'

    async def filter(self, update: UpdateNewMessage) -> Optional[Union[bool, dict]]:
        return isinstance(update, UpdateNewMessage)


class TextFilter(MessageFilter):
    name = 'Filters.text'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageText)


class AnimationFilter(MessageFilter):
    name = 'Filters.animation'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageAnimation)


class AudioFilter(MessageFilter):
    name = 'Filters.audio'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageAudio)


class ContactFilter(MessageFilter):
    name = 'Filters.contact'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageContact)


class DiceFilter(MessageFilter):
    name = 'Filters.dice'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageDice)


class DocumentFilter(MessageFilter):
    name = 'Filters.document'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageDocument)


class GameFilter(MessageFilter):
    name = 'Filters.game'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageGame)


class InvoiceFilter(MessageFilter):
    name = 'Filters.invoice'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageInvoice)


class LocationFilter(MessageFilter):
    name = 'Filters.location'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageLocation)


class PhotoFilter(MessageFilter):
    name = 'Filters.photo'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessagePhoto)


class PollFilter(MessageFilter):
    name = 'Filters.poll'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessagePoll)


class StickerFilter(MessageFilter):
    name = 'Filters.sticker'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageSticker)


class UnsupportedFilter(MessageFilter):
    name = 'Filters.unsupported'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageUnsupported)


class VenueFilter(MessageFilter):
    name = 'Filters.venue'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageVenue)


class VideoFilter(MessageFilter):
    name = 'Filters.video'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageVideo)


class VideoNoteFilter(MessageFilter):
    name = 'Filters.video_note'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageVideoNote)


class VoiceNoteFilter(MessageFilter):
    name = 'Filters.voice_note'

    async def filter(self, update: UpdateNewMessage) -> bool:
        return isinstance(update.message.content, MessageVoiceNote)


class BotCommandFilter(TextFilter):
    data_filter = True
    command: str = None

    def __init__(self, command: str = '*'):
        self.command = command

    async def filter(self, update: UpdateNewMessage) -> Union[bool, dict]:
        message_text = update.message.content.text.text

        if not message_text.startswith('/'):
            return False

        [bot_command, *bot_command_args] = message_text.lstrip('/').split()

        update.EXTRA['bot_command'] = bot_command
        update.EXTRA['bot_command_args'] = bot_command_args

        if not bool(self.command) or self.command in ["*", bot_command]:
            return {
                'bot_command': bot_command,
                'bot_command_args': bot_command_args
            }

        return False


class RegexFilter(TextFilter):
    data_filter = True
    regex: re.Pattern

    def __init__(self, pattern: str, flags: int = 0):
        self.regex = re.compile(pattern, flags=flags)

    async def filter(self, update: UpdateNewMessage) -> Union[bool, dict]:
        message_text = update.message.content.text.text
        match = self.regex.match(message_text)

        if bool(match):
            return {"regex_match": match}

        return False


class Filters:
    all = AllFilter()
    message = MessageFilter()
    text = TextFilter()
    audio = AudioFilter()
    contact = ContactFilter()
    dice = DiceFilter()
    document = DocumentFilter()
    game = GameFilter()
    invoice = InvoiceFilter()
    location = LocationFilter()
    photo = PhotoFilter()
    poll = PollFilter()
    sticker = StickerFilter()
    unsupported = UnsupportedFilter()
    venue = VenueFilter()
    video = VideoFilter()
    video_note = VideoNoteFilter()
    voice_note = VoiceNoteFilter()

    # "Callable" filters
    bot_command = BotCommandFilter
    regex = RegexFilter



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import InputSticker


class UploadStickerFile(BaseObject):
    """
    Uploads a PNG image with a sticker; returns the uploaded file
    
    Params:
        user_id (:class:`int`)
            Sticker file owner; ignored for regular users
        
        sticker (:class:`InputSticker`)
            Sticker file to upload
        
    """

    ID: str = Field("uploadStickerFile", alias="@type")
    user_id: int
    sticker: InputSticker

    @staticmethod
    def read(q: dict) -> UploadStickerFile:
        return UploadStickerFile.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetStickers(BaseObject):
    """
    Returns stickers from the installed sticker sets that correspond to a given emoji. If the emoji is not empty, favorite and recently used stickers may also be returned
    
    Params:
        emoji (:class:`str`)
            String representation of emoji. If empty, returns all known installed stickers
        
        limit (:class:`int`)
            The maximum number of stickers to be returned
        
    """

    ID: str = Field("getStickers", alias="@type")
    emoji: str
    limit: int

    @staticmethod
    def read(q: dict) -> GetStickers:
        return GetStickers.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetSuggestedStickerSetName(BaseObject):
    """
    Returns a suggested name for a new sticker set with a given title
    
    Params:
        title (:class:`str`)
            Sticker set title; 1-64 characters
        
    """

    ID: str = Field("getSuggestedStickerSetName", alias="@type")
    title: str = Field(..., min_length=1, max_length=64)

    @staticmethod
    def read(q: dict) -> GetSuggestedStickerSetName:
        return GetSuggestedStickerSetName.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class Address(BaseObject):
    """
    Describes an address
    
    Params:
        country_code (:class:`str`)
            A two-letter ISO 3166-1 alpha-2 country code
        
        state (:class:`str`)
            State, if applicable
        
        city (:class:`str`)
            City
        
        street_line1 (:class:`str`)
            First line of the address
        
        street_line2 (:class:`str`)
            Second line of the address
        
        postal_code (:class:`str`)
            Address postal code
        
    """

    ID: str = Field("address", alias="@type")
    country_code: str
    state: str
    city: str
    street_line1: str
    street_line2: str
    postal_code: str

    @staticmethod
    def read(q: dict) -> Address:
        return Address.construct(**q)



from __future__ import annotations

import logging
import typing

from sortedcontainers import SortedSet

from .api import (
    API,
    BasicGroup,
    BasicGroupFullInfo,
    Chat,
    ChatListMain,
    ChatPosition,
    OptionValueBoolean,
    OptionValueEmpty,
    OptionValueInteger,
    OptionValueString,
    SecretChat,
    Supergroup,
    SupergroupFullInfo,
    UpdateBasicGroup,
    UpdateBasicGroupFullInfo,
    UpdateChatDefaultDisableNotification,
    UpdateChatDraftMessage,
    UpdateChatHasScheduledMessages,
    UpdateChatIsBlocked,
    UpdateChatIsMarkedAsUnread,
    UpdateChatLastMessage,
    UpdateChatMessageTtlSetting,
    UpdateChatNotificationSettings,
    UpdateChatPermissions,
    UpdateChatPhoto,
    UpdateChatPosition,
    UpdateChatReadInbox,
    UpdateChatReadOutbox,
    UpdateChatReplyMarkup,
    UpdateChatTitle,
    UpdateChatUnreadMentionCount,
    UpdateChatVoiceChat,
    UpdateNewChat,
    UpdateOption,
    UpdateSecretChat,
    UpdateSupergroup,
    UpdateSupergroupFullInfo,
    UpdateUser,
    UpdateUserFullInfo,
    UpdateUserStatus,
    User,
    UserFullInfo,
)
from .tdjson import TDLIB_MAX_INT

if typing.TYPE_CHECKING:
    from .client import Client


class OrderedChat:
    chat_id: int
    position: ChatPosition

    def __init__(self, chat_id: int, chat_position: ChatPosition):
        self.chat_id = chat_id
        self.position = chat_position

    @property
    def order_tuple(self) -> tuple[int, int]:
        return self.position.order, self.chat_id

    def __hash__(self):
        return object.__hash__(self.chat_id)

    def __eq__(self, other: OrderedChat):
        return self.order_tuple == other.order_tuple

    def __ne__(self, other: OrderedChat):
        return not self.__eq__(other)

    def __lt__(self, other: OrderedChat):
        # Reverse order
        return self.order_tuple >= other.order_tuple

    def __gt__(self, other: OrderedChat):
        # Reverse order
        return self.order_tuple <= other.order_tuple

    def __le__(self, other: OrderedChat):
        # Reverse order
        return self.order_tuple > other.order_tuple

    def __ge__(self, other: OrderedChat):
        # Reverse order
        return self.order_tuple < other.order_tuple


class ClientCache:
    # Client Options
    options: dict[str, typing.Union[str, int, bool]] = {}

    users: dict[int, User] = {}
    basic_groups: dict[int, BasicGroup] = {}
    supergroups: dict[int, Supergroup] = {}
    secret_chats: dict[int, SecretChat] = {}

    # Full Info
    users_full_info: dict[int, UserFullInfo] = {}
    basic_groups_full_info: dict[int, BasicGroupFullInfo] = {}
    supergroups_full_info: dict[int, SupergroupFullInfo] = {}

    # Chats
    chats: dict[int, Chat] = {}
    main_chats_list: SortedSet[OrderedChat] = SortedSet()
    have_full_main_chats_list: bool = False

    def __init__(self, client: 'Client'):
        self.logger = logging.getLogger(__name__)
        self.client = client
        client.add_event_handler(
            self.__on_update_option,
            API.Types.UPDATE_OPTION
        )
        client.add_event_handler(
            self.__on_update_user,
            API.Types.UPDATE_USER
        )
        client.add_event_handler(
            self.__on_update_user_full_info,
            API.Types.UPDATE_USER_FULL_INFO
        )
        client.add_event_handler(
            self.__on_update_basic_group,
            API.Types.UPDATE_BASIC_GROUP
        )
        client.add_event_handler(
            self.__on_update_basic_group_full_info,
            API.Types.UPDATE_BASIC_GROUP_FULL_INFO
        )
        client.add_event_handler(
            self.__on_update_supergroup,
            API.Types.UPDATE_SUPERGROUP
        )
        client.add_event_handler(
            self.__on_update_supergroup_full_info,
            API.Types.UPDATE_SUPERGROUP_FULL_INFO
        )
        client.add_event_handler(
            self.__on_update_secret_chat,
            API.Types.UPDATE_SECRET_CHAT
        )
        client.add_event_handler(
            self.__on_update_new_chat,
            API.Types.UPDATE_NEW_CHAT
        )
        client.add_event_handler(
            self.__on_update_chat_position,
            API.Types.UPDATE_CHAT_POSITION
        )
        client.add_event_handler(
            self.__on_update_chat_last_message,
            API.Types.UPDATE_CHAT_LAST_MESSAGE
        )
        client.add_event_handler(
            self.__on_update_chat_draft_message,
            API.Types.UPDATE_CHAT_DRAFT_MESSAGE
        )
        client.add_event_handler(
            self.__on_update_chat_title,
            API.Types.UPDATE_CHAT_TITLE
        )
        client.add_event_handler(
            self.__on_update_chat_photo,
            API.Types.UPDATE_CHAT_PHOTO
        )
        client.add_event_handler(
            self.__on_update_chat_permissions,
            API.Types.UPDATE_CHAT_PERMISSIONS
        )
        client.add_event_handler(
            self.__on_update_chat_read_inbox,
            API.Types.UPDATE_CHAT_READ_INBOX
        )
        client.add_event_handler(
            self.__on_update_chat_read_outbox,
            API.Types.UPDATE_CHAT_READ_OUTBOX
        )
        client.add_event_handler(
            self.__on_update_chat_reply_markup,
            API.Types.UPDATE_CHAT_REPLY_MARKUP
        )
        client.add_event_handler(
            self.__on_update_chat_notification_settings,
            API.Types.UPDATE_CHAT_NOTIFICATION_SETTINGS
        )
        client.add_event_handler(
            self.__on_update_chat_unread_mention_count,
            API.Types.UPDATE_CHAT_UNREAD_MENTION_COUNT
        )
        client.add_event_handler(
            self.__on_update_chat_default_disable_notification,
            API.Types.UPDATE_CHAT_DEFAULT_DISABLE_NOTIFICATION
        )
        client.add_event_handler(
            self.__on_update_chat_is_blocked,
            API.Types.UPDATE_CHAT_IS_BLOCKED
        )
        client.add_event_handler(
            self.__on_update_chat_is_marked_as_unread,
            API.Types.UPDATE_CHAT_IS_MARKED_AS_UNREAD
        )
        client.add_event_handler(
            self.__on_update_chat_has_scheduled_messages,
            API.Types.UPDATE_CHAT_HAS_SCHEDULED_MESSAGES
        )
        client.add_event_handler(
            self.__on_update_chat_message_ttl_setting,
            API.Types.UPDATE_CHAT_MESSAGE_TTL_SETTING
        )
        client.add_event_handler(
            self.__on_update_chat_voice_chat,
            API.Types.UPDATE_CHAT_VOICE_CHAT
        )

    async def get_option_value(self, name: str) -> typing.Union[str, int, bool, None]:
        value = self.options.get(name)

        if not bool(value):
            option_value = await self.client.api.get_option()

            if isinstance(option_value, OptionValueEmpty):
                value = None
            elif isinstance(option_value, OptionValueInteger):
                value = int(option_value.value)
            elif isinstance(option_value, (OptionValueString, OptionValueBoolean,)):
                value = option_value.value
            else:
                value = None

            self.options[name] = value

        return value

    async def get_main_list_chats(self, limit: int = 25) -> list[Chat]:
        cached_chats_count = len(self.main_chats_list)

        if not self.have_full_main_chats_list and limit > cached_chats_count:
            offset_order = TDLIB_MAX_INT
            offset_chat_id = 0

            if cached_chats_count > 0:
                last_chat = self.main_chats_list[-1]
                offset_order = last_chat.position.order
                offset_chat_id = last_chat.chat_id

            request_limit = max(100, limit - cached_chats_count)
            result = await self.client.api.get_chats(ChatListMain(), offset_order, offset_chat_id, request_limit)

            if len(result.chat_ids) == 0:
                self.have_full_main_chats_list = True

            return await self.get_main_list_chats(limit)

        return [self.chats.get(oc.chat_id) for oc in self.main_chats_list[:limit]]

    async def get_chat(self, chat_id: int) -> Chat:
        chat = self.chats.get(chat_id)

        if not bool(chat):
            chat = await self.client.api.get_chat(chat_id)

        self.chats[chat_id] = chat
        self.__set_chat_positions(chat, chat.positions)
        return chat

    async def get_user(self, user_id: int) -> User:
        user = self.users.get(user_id)

        if not bool(user):
            user = await self.client.api.get_user(user_id)

        self.users[user_id] = user
        return user

    async def get_user_full_info(self, user_id: int) -> UserFullInfo:
        user_full_info = self.users_full_info.get(user_id)

        if not bool(user_full_info):
            user_full_info = await self.client.api.get_user_full_info(user_id)

        self.users_full_info[user_id] = user_full_info
        return user_full_info

    async def get_basic_group(self, basic_group_id: int) -> BasicGroup:
        basic_group = self.basic_groups.get(basic_group_id)

        if not bool(basic_group):
            basic_group = await self.client.api.get_basic_group(basic_group_id)

        self.basic_groups[basic_group_id] = basic_group
        return basic_group

    async def get_basic_group_full_info(self, basic_group_id: int) -> BasicGroupFullInfo:
        basic_group_full_info = self.basic_groups_full_info.get(basic_group_id)

        if not bool(basic_group_full_info):
            basic_group_full_info = await self.client.api.get_basic_group_full_info(basic_group_id)

        self.basic_groups_full_info[basic_group_id] = basic_group_full_info
        return basic_group_full_info

    async def get_supergroup(self, supergroup_id: int) -> Supergroup:
        supergroup = self.supergroups.get(supergroup_id)

        if not bool(supergroup):
            supergroup = await self.client.api.get_supergroup(supergroup_id)

        self.supergroups[supergroup_id] = supergroup
        return supergroup

    async def get_supergroup_full_info(self, supergroup_id: int) -> SupergroupFullInfo:
        supergroup_full_info = self.supergroups_full_info.get(supergroup_id)

        if not bool(supergroup_full_info):
            supergroup_full_info = await self.client.api.get_supergroup_full_info(supergroup_id)

        self.supergroups_full_info[supergroup_id] = supergroup_full_info
        return supergroup_full_info

    async def get_secret_chat(self, secret_chat_id: int) -> SecretChat:
        secret_chat = self.secret_chats.get(secret_chat_id)

        if not bool(secret_chat):
            secret_chat = await self.client.api.get_secret_chat(secret_chat_id)

        self.secret_chats[secret_chat_id] = secret_chat
        return secret_chat

    def __set_chat_positions(self, chat: Chat, positions: list[ChatPosition]):
        for position in positions:
            if isinstance(position.list, ChatListMain):
                try:
                    self.main_chats_list.remove(OrderedChat(chat.id, position))
                except (KeyError, ValueError):
                    pass

        chat.positions = positions

        for position in positions:
            if isinstance(position.list, ChatListMain):
                self.main_chats_list.add(OrderedChat(chat.id, position))

    async def __on_update_option(self, _: Client, update: UpdateOption):
        if isinstance(update.value, OptionValueEmpty):
            value = None
        elif isinstance(update.value, OptionValueInteger):
            value = int(update.value.value)
        elif isinstance(update.value, (OptionValueString, OptionValueBoolean,)):
            value = update.value.value
        else:
            return

        self.options[update.name] = value

    async def __on_update_user(self, _: Client, update: UpdateUser):
        self.users[update.user.id] = update.user

    async def __on_update_user_full_info(self, _: Client, update: UpdateUserFullInfo):
        self.users_full_info[update.user_id] = update.user_full_info

    async def __on_update_user_status(self, _: Client, update: UpdateUserStatus):
        self.users[update.user_id].status = update.status

    async def __on_update_basic_group(self, _: Client, update: UpdateBasicGroup):
        self.basic_groups[update.basic_group.id] = update.basic_group

    async def __on_update_basic_group_full_info(self, _: Client, update: UpdateBasicGroupFullInfo):
        self.basic_groups_full_info[update.basic_group_id] = update.basic_group_full_info

    async def __on_update_supergroup(self, _: Client, update: UpdateSupergroup):
        self.supergroups[update.supergroup.id] = update.supergroup

    async def __on_update_supergroup_full_info(self, _: Client, update: UpdateSupergroupFullInfo):
        self.supergroups_full_info[update.supergroup_id] = update.supergroup_full_info

    async def __on_update_secret_chat(self, _: Client, update: UpdateSecretChat):
        self.secret_chats[update.secret_chat.id] = update.secret_chat

    async def __on_update_new_chat(self, _: Client, update: UpdateNewChat):
        self.chats[update.chat.id] = update.chat
        self.__set_chat_positions(update.chat, update.chat.positions)

    async def __on_update_chat_position(self, _: Client, update: UpdateChatPosition):
        if isinstance(update.position.list, ChatListMain):
            chat = self.chats.get(update.chat_id)
            new_positions = [update.position] + [p for p in chat.positions if not isinstance(p, ChatListMain)]
            self.__set_chat_positions(chat, new_positions)

    async def __on_update_chat_last_message(self, _: Client, update: UpdateChatLastMessage):
        chat = self.chats.get(update.chat_id)
        chat.last_message = update.last_message
        self.__set_chat_positions(chat, update.positions)

    async def __on_update_chat_draft_message(self, _: Client, update: UpdateChatDraftMessage):
        chat = self.chats.get(update.chat_id)
        if not hasattr(update, 'draft_message'):
            print(update)
        chat.draft_message = update.draft_message
        self.__set_chat_positions(chat, update.positions)

    async def __on_update_chat_title(self, _: Client, update: UpdateChatTitle):
        self.chats[update.chat_id].title = update.title

    async def __on_update_chat_photo(self, _: Client, update: UpdateChatPhoto):
        self.chats[update.chat_id].photo = update.photo

    async def __on_update_chat_permissions(self, _: Client, update: UpdateChatPermissions):
        self.chats[update.chat_id].permissions = update.permissions

    async def __on_update_chat_read_inbox(self, _: Client, update: UpdateChatReadInbox):
        self.chats[update.chat_id].unread_count = update.unread_count
        self.chats[update.chat_id].last_read_inbox_message_id = update.last_read_inbox_message_id

    async def __on_update_chat_read_outbox(self, _: Client, update: UpdateChatReadOutbox):
        self.chats[update.chat_id].last_read_outbox_message_id = update.last_read_outbox_message_id

    async def __on_update_chat_reply_markup(self, _: Client, update: UpdateChatReplyMarkup):
        self.chats[update.chat_id].reply_markup_message_id = update.reply_markup_message_id

    async def __on_update_chat_message_ttl_setting(self, _: Client, update: UpdateChatMessageTtlSetting):
        self.chats[update.chat_id].message_ttl_setting = update.message_ttl_setting

    async def __on_update_chat_notification_settings(self, _: Client, update: UpdateChatNotificationSettings):
        self.chats[update.chat_id].notification_settings = update.notification_settings

    async def __on_update_chat_unread_mention_count(self, _: Client, update: UpdateChatUnreadMentionCount):
        self.chats[update.chat_id].unread_mention_count = update.unread_mention_count

    async def __on_update_chat_default_disable_notification(
            self,
            _: Client,
            update: UpdateChatDefaultDisableNotification
    ):
        self.chats[update.chat_id].default_disable_notification = update.default_disable_notification

    async def __on_update_chat_is_blocked(self, _: Client, update: UpdateChatIsBlocked):
        self.chats[update.chat_id].is_blocked = update.is_blocked

    async def __on_update_chat_is_marked_as_unread(self, _: Client, update: UpdateChatIsMarkedAsUnread):
        self.chats[update.chat_id].is_marked_as_unread = update.is_marked_as_unread

    async def __on_update_chat_has_scheduled_messages(self, _: Client, update: UpdateChatHasScheduledMessages):
        self.chats[update.chat_id].has_scheduled_messages = update.has_scheduled_messages

    async def __on_update_chat_voice_chat(self, _: Client, update: UpdateChatVoiceChat):
        self.chats[update.chat_id].voice_chat = update.voice_chat



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import ChatList
from ..types import SearchMessagesFilter


class SearchMessages(BaseObject):
    """
    Searches for messages in all chats except secret chats. Returns the results in reverse chronological order (i.e., in order of decreasing (date, chat_id, message_id)). For optimal performance the number of returned messages is chosen by the library
    
    Params:
        chat_list (:class:`ChatList`)
            Chat list in which to search messages; pass null to search in all chats regardless of their chat list. Only Main and Archive chat lists are supported
        
        query (:class:`str`)
            Query to search for
        
        offset_date (:class:`int`)
            The date of the message starting from which the results should be fetched. Use 0 or any date in the future to get results from the last message
        
        offset_chat_id (:class:`int`)
            The chat identifier of the last found message, or 0 for the first request
        
        offset_message_id (:class:`int`)
            The message identifier of the last found message, or 0 for the first request
        
        limit (:class:`int`)
            The maximum number of messages to be returned; up to 100. Fewer messages may be returned than specified by the limit, even if the end of the message history has not been reached
        
        filter_ (:class:`SearchMessagesFilter`)
            Filter for message content in the search results; searchMessagesFilterCall, searchMessagesFilterMissedCall, searchMessagesFilterMention, searchMessagesFilterUnreadMention, searchMessagesFilterFailedToSend and searchMessagesFilterPinned are unsupported in this function
        
        min_date (:class:`int`)
            If not 0, the minimum date of the messages to return
        
        max_date (:class:`int`)
            If not 0, the maximum date of the messages to return
        
    """

    ID: str = Field("searchMessages", alias="@type")
    chat_list: ChatList
    query: str
    offset_date: int
    offset_chat_id: int
    offset_message_id: int
    limit: int
    filter_: SearchMessagesFilter = Field(..., alias='filter')
    min_date: int
    max_date: int

    @staticmethod
    def read(q: dict) -> SearchMessages:
        return SearchMessages.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class SetLogVerbosityLevel(BaseObject):
    """
    Sets the verbosity level of the internal logging of TDLib. Can be called synchronously
    
    Params:
        new_verbosity_level (:class:`int`)
            New value of the verbosity level for logging. Value 0 corresponds to fatal errors, value 1 corresponds to errors, value 2 corresponds to warnings and debug warnings, value 3 corresponds to informational, value 4 corresponds to debug, value 5 corresponds to verbose debug, value greater than 5 and up to 1023 can be used to enable even more logging
        
    """

    ID: str = Field("setLogVerbosityLevel", alias="@type")
    new_verbosity_level: int

    @staticmethod
    def read(q: dict) -> SetLogVerbosityLevel:
        return SetLogVerbosityLevel.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .address import Address
from ..base_object import BaseObject


class OrderInfo(BaseObject):
    """
    Order information
    
    Params:
        name (:class:`str`)
            Name of the user
        
        phone_number (:class:`str`)
            Phone number of the user
        
        email_address (:class:`str`)
            Email address of the user
        
        shipping_address (:class:`Address`)
            Shipping address for this order; may be null
        
    """

    ID: str = Field("orderInfo", alias="@type")
    name: str
    phone_number: str
    email_address: str
    shipping_address: typing.Optional[Address] = None

    @staticmethod
    def read(q: dict) -> OrderInfo:
        return OrderInfo.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import LanguagePackInfo


class EditCustomLanguagePackInfo(BaseObject):
    """
    Edits information about a custom local language pack in the current localization target. Can be called before authorization
    
    Params:
        info (:class:`LanguagePackInfo`)
            New information about the custom local language pack
        
    """

    ID: str = Field("editCustomLanguagePackInfo", alias="@type")
    info: LanguagePackInfo

    @staticmethod
    def read(q: dict) -> EditCustomLanguagePackInfo:
        return EditCustomLanguagePackInfo.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import InputCredentials


class SendPaymentForm(BaseObject):
    """
    Sends a filled-out payment form to the bot for final verification
    
    Params:
        chat_id (:class:`int`)
            Chat identifier of the Invoice message
        
        message_id (:class:`int`)
            Message identifier
        
        payment_form_id (:class:`int`)
            Payment form identifier returned by getPaymentForm
        
        order_info_id (:class:`str`)
            Identifier returned by validateOrderInfo, or an empty string
        
        shipping_option_id (:class:`str`)
            Identifier of a chosen shipping option, if applicable
        
        credentials (:class:`InputCredentials`)
            The credentials chosen by user for payment
        
        tip_amount (:class:`int`)
            Chosen by the user amount of tip in the smallest units of the currency
        
    """

    ID: str = Field("sendPaymentForm", alias="@type")
    chat_id: int
    message_id: int
    payment_form_id: int
    order_info_id: str
    shipping_option_id: str
    credentials: InputCredentials
    tip_amount: int

    @staticmethod
    def read(q: dict) -> SendPaymentForm:
        return SendPaymentForm.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import MessageSender


class ToggleGroupCallParticipantIsMuted(BaseObject):
    """
    Toggles whether a participant of an active group call is muted, unmuted, or allowed to unmute themselves
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier
        
        participant_id (:class:`MessageSender`)
            Participant identifier
        
        is_muted (:class:`bool`)
            Pass true if the user must be muted and false otherwise
        
    """

    ID: str = Field("toggleGroupCallParticipantIsMuted", alias="@type")
    group_call_id: int
    participant_id: MessageSender
    is_muted: bool

    @staticmethod
    def read(q: dict) -> ToggleGroupCallParticipantIsMuted:
        return ToggleGroupCallParticipantIsMuted.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .message_sender import MessageSender
from ..base_object import BaseObject


class VoiceChat(BaseObject):
    """
    Describes a voice chat
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier of an active voice chat; 0 if none. Full informationa about the voice chat can be received through the method getGroupCall
        
        has_participants (:class:`bool`)
            True, if the voice chat has participants
        
        default_participant_id (:class:`MessageSender`)
            Default group call participant identifier to join the voice chat; may be null
        
    """

    ID: str = Field("voiceChat", alias="@type")
    group_call_id: int
    has_participants: bool
    default_participant_id: typing.Optional[MessageSender] = None

    @staticmethod
    def read(q: dict) -> VoiceChat:
        return VoiceChat.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetCountries(BaseObject):
    """
    Returns information about existing countries. Can be called before authorization
    
    """

    ID: str = Field("getCountries", alias="@type")

    @staticmethod
    def read(q: dict) -> GetCountries:
        return GetCountries.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class ToggleGroupCallEnabledStartNotification(BaseObject):
    """
    Toggles whether the current user will receive a notification when the group call will start; scheduled group calls only
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier
        
        enabled_start_notification (:class:`bool`)
            New value of the enabled_start_notification setting
        
    """

    ID: str = Field("toggleGroupCallEnabledStartNotification", alias="@type")
    group_call_id: int
    enabled_start_notification: bool

    @staticmethod
    def read(q: dict) -> ToggleGroupCallEnabledStartNotification:
        return ToggleGroupCallEnabledStartNotification.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .animation import Animation
from .audio import Audio
from .chat_photo_info import ChatPhotoInfo
from .location import Location
from .page_block_caption import PageBlockCaption
from .page_block_related_article import PageBlockRelatedArticle
from .page_block_table_cell import PageBlockTableCell
from .photo import Photo
from .rich_text import RichText
from .video import Video
from .voice_note import VoiceNote
from ..base_object import BaseObject


class PageBlock(BaseObject):
    """
    Describes a block of an instant view web page
    
    """

    ID: str = Field("pageBlock", alias="@type")


class PageBlockAnchor(PageBlock):
    """
    An invisible anchor on a page, which can be used in a URL to open the page from the specified anchor
    
    Params:
        name (:class:`str`)
            Name of the anchor
        
    """

    ID: str = Field("pageBlockAnchor", alias="@type")
    name: str

    @staticmethod
    def read(q: dict) -> PageBlockAnchor:
        return PageBlockAnchor.construct(**q)


class PageBlockAnimation(PageBlock):
    """
    An animation
    
    Params:
        animation (:class:`Animation`)
            Animation file; may be null
        
        caption (:class:`PageBlockCaption`)
            Animation caption
        
        need_autoplay (:class:`bool`)
            True, if the animation should be played automatically
        
    """

    ID: str = Field("pageBlockAnimation", alias="@type")
    animation: typing.Optional[Animation] = None
    caption: PageBlockCaption
    need_autoplay: bool

    @staticmethod
    def read(q: dict) -> PageBlockAnimation:
        return PageBlockAnimation.construct(**q)


class PageBlockAudio(PageBlock):
    """
    An audio file
    
    Params:
        audio (:class:`Audio`)
            Audio file; may be null
        
        caption (:class:`PageBlockCaption`)
            Audio file caption
        
    """

    ID: str = Field("pageBlockAudio", alias="@type")
    audio: typing.Optional[Audio] = None
    caption: PageBlockCaption

    @staticmethod
    def read(q: dict) -> PageBlockAudio:
        return PageBlockAudio.construct(**q)


class PageBlockAuthorDate(PageBlock):
    """
    The author and publishing date of a page
    
    Params:
        author (:class:`RichText`)
            Author
        
        publish_date (:class:`int`)
            Point in time (Unix timestamp) when the article was published; 0 if unknown
        
    """

    ID: str = Field("pageBlockAuthorDate", alias="@type")
    author: RichText
    publish_date: int

    @staticmethod
    def read(q: dict) -> PageBlockAuthorDate:
        return PageBlockAuthorDate.construct(**q)


class PageBlockBlockQuote(PageBlock):
    """
    A block quote
    
    Params:
        text (:class:`RichText`)
            Quote text
        
        credit (:class:`RichText`)
            Quote credit
        
    """

    ID: str = Field("pageBlockBlockQuote", alias="@type")
    text: RichText
    credit: RichText

    @staticmethod
    def read(q: dict) -> PageBlockBlockQuote:
        return PageBlockBlockQuote.construct(**q)


class PageBlockChatLink(PageBlock):
    """
    A link to a chat
    
    Params:
        title (:class:`str`)
            Chat title
        
        photo (:class:`ChatPhotoInfo`)
            Chat photo; may be null
        
        username (:class:`str`)
            Chat username, by which all other information about the chat should be resolved
        
    """

    ID: str = Field("pageBlockChatLink", alias="@type")
    title: str
    photo: typing.Optional[ChatPhotoInfo] = None
    username: str

    @staticmethod
    def read(q: dict) -> PageBlockChatLink:
        return PageBlockChatLink.construct(**q)


class PageBlockCollage(PageBlock):
    """
    A collage
    
    Params:
        page_blocks (:obj:`list[PageBlock]`)
            Collage item contents
        
        caption (:class:`PageBlockCaption`)
            Block caption
        
    """

    ID: str = Field("pageBlockCollage", alias="@type")
    page_blocks: list[PageBlock]
    caption: PageBlockCaption

    @staticmethod
    def read(q: dict) -> PageBlockCollage:
        return PageBlockCollage.construct(**q)


class PageBlockCover(PageBlock):
    """
    A page cover
    
    Params:
        cover (:class:`PageBlock`)
            Cover
        
    """

    ID: str = Field("pageBlockCover", alias="@type")
    cover: PageBlock

    @staticmethod
    def read(q: dict) -> PageBlockCover:
        return PageBlockCover.construct(**q)


class PageBlockDetails(PageBlock):
    """
    A collapsible block
    
    Params:
        header (:class:`RichText`)
            Always visible heading for the block
        
        page_blocks (:obj:`list[PageBlock]`)
            Block contents
        
        is_open (:class:`bool`)
            True, if the block is open by default
        
    """

    ID: str = Field("pageBlockDetails", alias="@type")
    header: RichText
    page_blocks: list[PageBlock]
    is_open: bool

    @staticmethod
    def read(q: dict) -> PageBlockDetails:
        return PageBlockDetails.construct(**q)


class PageBlockDivider(PageBlock):
    """
    An empty block separating a page
    
    """

    ID: str = Field("pageBlockDivider", alias="@type")

    @staticmethod
    def read(q: dict) -> PageBlockDivider:
        return PageBlockDivider.construct(**q)


class PageBlockEmbedded(PageBlock):
    """
    An embedded web page
    
    Params:
        url (:class:`str`)
            Web page URL, if available
        
        html (:class:`str`)
            HTML-markup of the embedded page
        
        poster_photo (:class:`Photo`)
            Poster photo, if available; may be null
        
        width (:class:`int`)
            Block width; 0 if unknown
        
        height (:class:`int`)
            Block height; 0 if unknown
        
        caption (:class:`PageBlockCaption`)
            Block caption
        
        is_full_width (:class:`bool`)
            True, if the block should be full width
        
        allow_scrolling (:class:`bool`)
            True, if scrolling should be allowed
        
    """

    ID: str = Field("pageBlockEmbedded", alias="@type")
    url: str
    html: str
    poster_photo: typing.Optional[Photo] = None
    width: int
    height: int
    caption: PageBlockCaption
    is_full_width: bool
    allow_scrolling: bool

    @staticmethod
    def read(q: dict) -> PageBlockEmbedded:
        return PageBlockEmbedded.construct(**q)


class PageBlockEmbeddedPost(PageBlock):
    """
    An embedded post
    
    Params:
        url (:class:`str`)
            Web page URL
        
        author (:class:`str`)
            Post author
        
        author_photo (:class:`Photo`)
            Post author photo; may be null
        
        date (:class:`int`)
            Point in time (Unix timestamp) when the post was created; 0 if unknown
        
        page_blocks (:obj:`list[PageBlock]`)
            Post content
        
        caption (:class:`PageBlockCaption`)
            Post caption
        
    """

    ID: str = Field("pageBlockEmbeddedPost", alias="@type")
    url: str
    author: str
    author_photo: typing.Optional[Photo] = None
    date: int
    page_blocks: list[PageBlock]
    caption: PageBlockCaption

    @staticmethod
    def read(q: dict) -> PageBlockEmbeddedPost:
        return PageBlockEmbeddedPost.construct(**q)


class PageBlockFooter(PageBlock):
    """
    The footer of a page
    
    Params:
        footer (:class:`RichText`)
            Footer
        
    """

    ID: str = Field("pageBlockFooter", alias="@type")
    footer: RichText

    @staticmethod
    def read(q: dict) -> PageBlockFooter:
        return PageBlockFooter.construct(**q)


class PageBlockHeader(PageBlock):
    """
    A header
    
    Params:
        header (:class:`RichText`)
            Header
        
    """

    ID: str = Field("pageBlockHeader", alias="@type")
    header: RichText

    @staticmethod
    def read(q: dict) -> PageBlockHeader:
        return PageBlockHeader.construct(**q)


class PageBlockKicker(PageBlock):
    """
    A kicker
    
    Params:
        kicker (:class:`RichText`)
            Kicker
        
    """

    ID: str = Field("pageBlockKicker", alias="@type")
    kicker: RichText

    @staticmethod
    def read(q: dict) -> PageBlockKicker:
        return PageBlockKicker.construct(**q)


class PageBlockList(PageBlock):
    """
    A list of data blocks
    
    Params:
        items (:obj:`list[PageBlockListItem]`)
            The items of the list
        
    """

    ID: str = Field("pageBlockList", alias="@type")
    items: list[PageBlockListItem]

    @staticmethod
    def read(q: dict) -> PageBlockList:
        return PageBlockList.construct(**q)


class PageBlockMap(PageBlock):
    """
    A map
    
    Params:
        location (:class:`Location`)
            Location of the map center
        
        zoom (:class:`int`)
            Map zoom level
        
        width (:class:`int`)
            Map width
        
        height (:class:`int`)
            Map height
        
        caption (:class:`PageBlockCaption`)
            Block caption
        
    """

    ID: str = Field("pageBlockMap", alias="@type")
    location: Location
    zoom: int
    width: int
    height: int
    caption: PageBlockCaption

    @staticmethod
    def read(q: dict) -> PageBlockMap:
        return PageBlockMap.construct(**q)


class PageBlockParagraph(PageBlock):
    """
    A text paragraph
    
    Params:
        text (:class:`RichText`)
            Paragraph text
        
    """

    ID: str = Field("pageBlockParagraph", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> PageBlockParagraph:
        return PageBlockParagraph.construct(**q)


class PageBlockPhoto(PageBlock):
    """
    A photo
    
    Params:
        photo (:class:`Photo`)
            Photo file; may be null
        
        caption (:class:`PageBlockCaption`)
            Photo caption
        
        url (:class:`str`)
            URL that needs to be opened when the photo is clicked
        
    """

    ID: str = Field("pageBlockPhoto", alias="@type")
    photo: typing.Optional[Photo] = None
    caption: PageBlockCaption
    url: str

    @staticmethod
    def read(q: dict) -> PageBlockPhoto:
        return PageBlockPhoto.construct(**q)


class PageBlockPreformatted(PageBlock):
    """
    A preformatted text paragraph
    
    Params:
        text (:class:`RichText`)
            Paragraph text
        
        language (:class:`str`)
            Programming language for which the text should be formatted
        
    """

    ID: str = Field("pageBlockPreformatted", alias="@type")
    text: RichText
    language: str

    @staticmethod
    def read(q: dict) -> PageBlockPreformatted:
        return PageBlockPreformatted.construct(**q)


class PageBlockPullQuote(PageBlock):
    """
    A pull quote
    
    Params:
        text (:class:`RichText`)
            Quote text
        
        credit (:class:`RichText`)
            Quote credit
        
    """

    ID: str = Field("pageBlockPullQuote", alias="@type")
    text: RichText
    credit: RichText

    @staticmethod
    def read(q: dict) -> PageBlockPullQuote:
        return PageBlockPullQuote.construct(**q)


class PageBlockRelatedArticles(PageBlock):
    """
    Related articles
    
    Params:
        header (:class:`RichText`)
            Block header
        
        articles (:obj:`list[PageBlockRelatedArticle]`)
            List of related articles
        
    """

    ID: str = Field("pageBlockRelatedArticles", alias="@type")
    header: RichText
    articles: list[PageBlockRelatedArticle]

    @staticmethod
    def read(q: dict) -> PageBlockRelatedArticles:
        return PageBlockRelatedArticles.construct(**q)


class PageBlockSlideshow(PageBlock):
    """
    A slideshow
    
    Params:
        page_blocks (:obj:`list[PageBlock]`)
            Slideshow item contents
        
        caption (:class:`PageBlockCaption`)
            Block caption
        
    """

    ID: str = Field("pageBlockSlideshow", alias="@type")
    page_blocks: list[PageBlock]
    caption: PageBlockCaption

    @staticmethod
    def read(q: dict) -> PageBlockSlideshow:
        return PageBlockSlideshow.construct(**q)


class PageBlockSubheader(PageBlock):
    """
    A subheader
    
    Params:
        subheader (:class:`RichText`)
            Subheader
        
    """

    ID: str = Field("pageBlockSubheader", alias="@type")
    subheader: RichText

    @staticmethod
    def read(q: dict) -> PageBlockSubheader:
        return PageBlockSubheader.construct(**q)


class PageBlockSubtitle(PageBlock):
    """
    The subtitle of a page
    
    Params:
        subtitle (:class:`RichText`)
            Subtitle
        
    """

    ID: str = Field("pageBlockSubtitle", alias="@type")
    subtitle: RichText

    @staticmethod
    def read(q: dict) -> PageBlockSubtitle:
        return PageBlockSubtitle.construct(**q)


class PageBlockTable(PageBlock):
    """
    A table
    
    Params:
        caption (:class:`RichText`)
            Table caption
        
        cells (:obj:`list[list[PageBlockTableCell]]`)
            Table cells
        
        is_bordered (:class:`bool`)
            True, if the table is bordered
        
        is_striped (:class:`bool`)
            True, if the table is striped
        
    """

    ID: str = Field("pageBlockTable", alias="@type")
    caption: RichText
    cells: list[list[PageBlockTableCell]]
    is_bordered: bool
    is_striped: bool

    @staticmethod
    def read(q: dict) -> PageBlockTable:
        return PageBlockTable.construct(**q)


class PageBlockTitle(PageBlock):
    """
    The title of a page
    
    Params:
        title (:class:`RichText`)
            Title
        
    """

    ID: str = Field("pageBlockTitle", alias="@type")
    title: RichText

    @staticmethod
    def read(q: dict) -> PageBlockTitle:
        return PageBlockTitle.construct(**q)


class PageBlockVideo(PageBlock):
    """
    A video
    
    Params:
        video (:class:`Video`)
            Video file; may be null
        
        caption (:class:`PageBlockCaption`)
            Video caption
        
        need_autoplay (:class:`bool`)
            True, if the video should be played automatically
        
        is_looped (:class:`bool`)
            True, if the video should be looped
        
    """

    ID: str = Field("pageBlockVideo", alias="@type")
    video: typing.Optional[Video] = None
    caption: PageBlockCaption
    need_autoplay: bool
    is_looped: bool

    @staticmethod
    def read(q: dict) -> PageBlockVideo:
        return PageBlockVideo.construct(**q)


class PageBlockVoiceNote(PageBlock):
    """
    A voice note
    
    Params:
        voice_note (:class:`VoiceNote`)
            Voice note; may be null
        
        caption (:class:`PageBlockCaption`)
            Voice note caption
        
    """

    ID: str = Field("pageBlockVoiceNote", alias="@type")
    voice_note: typing.Optional[VoiceNote] = None
    caption: PageBlockCaption

    @staticmethod
    def read(q: dict) -> PageBlockVoiceNote:
        return PageBlockVoiceNote.construct(**q)


class PageBlockListItem(BaseObject):
    """
    Describes an item of a list page block
    
    Params:
        label (:class:`str`)
            Item label
        
        page_blocks (:obj:`list[PageBlock]`)
            Item blocks
        
    """

    ID: str = Field("pageBlockListItem", alias="@type")
    label: str
    page_blocks: list[PageBlock]

    @staticmethod
    def read(q: dict) -> PageBlockListItem:
        return PageBlockListItem.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class SetUsername(BaseObject):
    """
    Changes the username of the current user
    
    Params:
        username (:class:`str`)
            The new value of the username. Use an empty string to remove the username
        
    """

    ID: str = Field("setUsername", alias="@type")
    username: str

    @staticmethod
    def read(q: dict) -> SetUsername:
        return SetUsername.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .network_statistics_entry import NetworkStatisticsEntry
from ..base_object import BaseObject


class NetworkStatistics(BaseObject):
    """
    A full list of available network statistic entries
    
    Params:
        since_date (:class:`int`)
            Point in time (Unix timestamp) from which the statistics are collected
        
        entries (:obj:`list[NetworkStatisticsEntry]`)
            Network statistics entries
        
    """

    ID: str = Field("networkStatistics", alias="@type")
    since_date: int
    entries: list[NetworkStatisticsEntry]

    @staticmethod
    def read(q: dict) -> NetworkStatistics:
        return NetworkStatistics.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import FileType


class OptimizeStorage(BaseObject):
    """
    Optimizes storage usage, i.e. deletes some files and returns new storage usage statistics. Secret thumbnails can't be deleted
    
    Params:
        size (:class:`int`)
            Limit on the total size of files after deletion. Pass -1 to use the default limit
        
        ttl (:class:`int`)
            Limit on the time that has passed since the last time a file was accessed (or creation time for some filesystems). Pass -1 to use the default limit
        
        count (:class:`int`)
            Limit on the total count of files after deletion. Pass -1 to use the default limit
        
        immunity_delay (:class:`int`)
            The amount of time after the creation of a file during which it can't be deleted, in seconds. Pass -1 to use the default value
        
        file_types (:obj:`list[FileType]`)
            If not empty, only files with the given type(s) are considered. By default, all types except thumbnails, profile photos, stickers and wallpapers are deleted
        
        chat_ids (:obj:`list[int]`)
            If not empty, only files from the given chats are considered. Use 0 as chat identifier to delete files not belonging to any chat (e.g., profile photos)
        
        exclude_chat_ids (:obj:`list[int]`)
            If not empty, files from the given chats are excluded. Use 0 as chat identifier to exclude all files not belonging to any chat (e.g., profile photos)
        
        return_deleted_file_statistics (:class:`bool`)
            Pass true if statistics about the files that were deleted must be returned instead of the whole storage usage statistics. Affects only returned statistics
        
        chat_limit (:class:`int`)
            Same as in getStorageStatistics. Affects only returned statistics
        
    """

    ID: str = Field("optimizeStorage", alias="@type")
    size: int
    ttl: int
    count: int
    immunity_delay: int
    file_types: list[FileType]
    chat_ids: list[int]
    exclude_chat_ids: list[int]
    return_deleted_file_statistics: bool
    chat_limit: int

    @staticmethod
    def read(q: dict) -> OptimizeStorage:
        return OptimizeStorage.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class CancelDownloadFile(BaseObject):
    """
    Stops the downloading of a file. If a file has already been downloaded, does nothing
    
    Params:
        file_id (:class:`int`)
            Identifier of a file to stop downloading
        
        only_if_pending (:class:`bool`)
            Pass true to stop downloading only if it hasn't been started, i.e. request hasn't been sent to server
        
    """

    ID: str = Field("cancelDownloadFile", alias="@type")
    file_id: int
    only_if_pending: bool

    @staticmethod
    def read(q: dict) -> CancelDownloadFile:
        return CancelDownloadFile.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class RecoverAuthenticationPassword(BaseObject):
    """
    Recovers the password with a password recovery code sent to an email address that was previously set up. Works only when the current authorization state is authorizationStateWaitPassword
    
    Params:
        recovery_code (:class:`str`)
            Recovery code to check
        
        new_password (:class:`str`)
            New password of the user; may be empty to remove the password
        
        new_hint (:class:`str`)
            New password hint; may be empty
        
    """

    ID: str = Field("recoverAuthenticationPassword", alias="@type")
    recovery_code: str
    new_password: str
    new_hint: str

    @staticmethod
    def read(q: dict) -> RecoverAuthenticationPassword:
        return RecoverAuthenticationPassword.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .message_sender import MessageSender
from ..base_object import BaseObject


class MessageReplyInfo(BaseObject):
    """
    Contains information about replies to a message
    
    Params:
        reply_count (:class:`int`)
            Number of times the message was directly or indirectly replied
        
        recent_repliers (:obj:`list[MessageSender]`)
            Recent repliers to the message; available in channels with a discussion supergroup
        
        last_read_inbox_message_id (:class:`int`)
            Identifier of the last read incoming reply to the message
        
        last_read_outbox_message_id (:class:`int`)
            Identifier of the last read outgoing reply to the message
        
        last_message_id (:class:`int`)
            Identifier of the last reply to the message
        
    """

    ID: str = Field("messageReplyInfo", alias="@type")
    reply_count: int
    recent_repliers: list[MessageSender]
    last_read_inbox_message_id: int
    last_read_outbox_message_id: int
    last_message_id: int

    @staticmethod
    def read(q: dict) -> MessageReplyInfo:
        return MessageReplyInfo.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class ThumbnailFormat(BaseObject):
    """
    Describes format of the thumbnail
    
    """

    ID: str = Field("thumbnailFormat", alias="@type")


class ThumbnailFormatGif(ThumbnailFormat):
    """
    The thumbnail is in static GIF format. It will be used only for some bot inline results
    
    """

    ID: str = Field("thumbnailFormatGif", alias="@type")

    @staticmethod
    def read(q: dict) -> ThumbnailFormatGif:
        return ThumbnailFormatGif.construct(**q)


class ThumbnailFormatJpeg(ThumbnailFormat):
    """
    The thumbnail is in JPEG format
    
    """

    ID: str = Field("thumbnailFormatJpeg", alias="@type")

    @staticmethod
    def read(q: dict) -> ThumbnailFormatJpeg:
        return ThumbnailFormatJpeg.construct(**q)


class ThumbnailFormatMpeg4(ThumbnailFormat):
    """
    The thumbnail is in MPEG4 format. It will be used only for some animations and videos
    
    """

    ID: str = Field("thumbnailFormatMpeg4", alias="@type")

    @staticmethod
    def read(q: dict) -> ThumbnailFormatMpeg4:
        return ThumbnailFormatMpeg4.construct(**q)


class ThumbnailFormatPng(ThumbnailFormat):
    """
    The thumbnail is in PNG format. It will be used only for background patterns
    
    """

    ID: str = Field("thumbnailFormatPng", alias="@type")

    @staticmethod
    def read(q: dict) -> ThumbnailFormatPng:
        return ThumbnailFormatPng.construct(**q)


class ThumbnailFormatTgs(ThumbnailFormat):
    """
    The thumbnail is in TGS format. It will be used only for animated sticker sets
    
    """

    ID: str = Field("thumbnailFormatTgs", alias="@type")

    @staticmethod
    def read(q: dict) -> ThumbnailFormatTgs:
        return ThumbnailFormatTgs.construct(**q)


class ThumbnailFormatWebp(ThumbnailFormat):
    """
    The thumbnail is in WEBP format. It will be used only for some stickers
    
    """

    ID: str = Field("thumbnailFormatWebp", alias="@type")

    @staticmethod
    def read(q: dict) -> ThumbnailFormatWebp:
        return ThumbnailFormatWebp.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import ProxyType


class EditProxy(BaseObject):
    """
    Edits an existing proxy server for network requests. Can be called before authorization
    
    Params:
        proxy_id (:class:`int`)
            Proxy identifier
        
        server (:class:`str`)
            Proxy server IP address
        
        port (:class:`int`)
            Proxy server port
        
        enable (:class:`bool`)
            True, if the proxy should be enabled
        
        type_ (:class:`ProxyType`)
            Proxy type
        
    """

    ID: str = Field("editProxy", alias="@type")
    proxy_id: int
    server: str
    port: int
    enable: bool
    type_: ProxyType = Field(..., alias='type')

    @staticmethod
    def read(q: dict) -> EditProxy:
        return EditProxy.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class InputFile(BaseObject):
    """
    Points to a file
    
    """

    ID: str = Field("inputFile", alias="@type")


class InputFileGenerated(InputFile):
    """
    A file generated by the application
    
    Params:
        original_path (:class:`str`)
            Local path to a file from which the file is generated; may be empty if there is no such file
        
        conversion (:class:`str`)
            String specifying the conversion applied to the original file; should be persistent across application restarts. Conversions beginning with '#' are reserved for internal TDLib usage
        
        expected_size (:class:`int`)
            Expected size of the generated file; 0 if unknown
        
    """

    ID: str = Field("inputFileGenerated", alias="@type")
    original_path: str
    conversion: str
    expected_size: int

    @staticmethod
    def read(q: dict) -> InputFileGenerated:
        return InputFileGenerated.construct(**q)


class InputFileId(InputFile):
    """
    A file defined by its unique ID
    
    Params:
        id (:class:`int`)
            Unique file identifier
        
    """

    ID: str = Field("inputFileId", alias="@type")
    id: int

    @staticmethod
    def read(q: dict) -> InputFileId:
        return InputFileId.construct(**q)


class InputFileLocal(InputFile):
    """
    A file defined by a local path
    
    Params:
        path (:class:`str`)
            Local path to the file
        
    """

    ID: str = Field("inputFileLocal", alias="@type")
    path: str

    @staticmethod
    def read(q: dict) -> InputFileLocal:
        return InputFileLocal.construct(**q)


class InputFileRemote(InputFile):
    """
    A file defined by its remote ID. The remote ID is guaranteed to be usable only if the corresponding file is still accessible to the user and known to TDLib. For example, if the file is from a message, then the message must be not deleted and accessible to the user. If the file database is disabled, then the corresponding object with the file must be preloaded by the application
    
    Params:
        id (:class:`str`)
            Remote file identifier
        
    """

    ID: str = Field("inputFileRemote", alias="@type")
    id: str

    @staticmethod
    def read(q: dict) -> InputFileRemote:
        return InputFileRemote.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import MessageSender


class ToggleGroupCallParticipantIsHandRaised(BaseObject):
    """
    Toggles whether a group call participant hand is rased
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier
        
        participant_id (:class:`MessageSender`)
            Participant identifier
        
        is_hand_raised (:class:`bool`)
            Pass true if the user's hand should be raised. Only self hand can be raised. Requires groupCall.can_be_managed group call flag to lower other's hand
        
    """

    ID: str = Field("toggleGroupCallParticipantIsHandRaised", alias="@type")
    group_call_id: int
    participant_id: MessageSender
    is_hand_raised: bool

    @staticmethod
    def read(q: dict) -> ToggleGroupCallParticipantIsHandRaised:
        return ToggleGroupCallParticipantIsHandRaised.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .background_type import BackgroundType
from .document import Document
from ..base_object import BaseObject


class Background(BaseObject):
    """
    Describes a chat background
    
    Params:
        id (:class:`int`)
            Unique background identifier
        
        is_default (:class:`bool`)
            True, if this is one of default backgrounds
        
        is_dark (:class:`bool`)
            True, if the background is dark and is recommended to be used with dark theme
        
        name (:class:`str`)
            Unique background name
        
        document (:class:`Document`)
            Document with the background; may be null. Null only for filled backgrounds
        
        type_ (:class:`BackgroundType`)
            Type of the background
        
    """

    ID: str = Field("background", alias="@type")
    id: int
    is_default: bool
    is_dark: bool
    name: str
    document: typing.Optional[Document] = None
    type_: BackgroundType = Field(..., alias='type')

    @staticmethod
    def read(q: dict) -> Background:
        return Background.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import LanguagePackString


class SetCustomLanguagePackString(BaseObject):
    """
    Adds, edits or deletes a string in a custom local language pack. Can be called before authorization
    
    Params:
        language_pack_id (:class:`str`)
            Identifier of a previously added custom local language pack in the current localization target
        
        new_string (:class:`LanguagePackString`)
            New language pack string
        
    """

    ID: str = Field("setCustomLanguagePackString", alias="@type")
    language_pack_id: str
    new_string: LanguagePackString

    @staticmethod
    def read(q: dict) -> SetCustomLanguagePackString:
        return SetCustomLanguagePackString.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .chat_filter import ChatFilter
from ..base_object import BaseObject


class RecommendedChatFilter(BaseObject):
    """
    Describes a recommended chat filter
    
    Params:
        filter_ (:class:`ChatFilter`)
            The chat filter
        
        param_description (:class:`str`)
            Chat filter description
        
    """

    ID: str = Field("recommendedChatFilter", alias="@type")
    filter_: ChatFilter = Field(..., alias='filter')
    param_description: str

    @staticmethod
    def read(q: dict) -> RecommendedChatFilter:
        return RecommendedChatFilter.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import MessageSendOptions


class SendInlineQueryResultMessage(BaseObject):
    """
    Sends the result of an inline query as a message. Returns the sent message. Always clears a chat draft message
    
    Params:
        chat_id (:class:`int`)
            Target chat
        
        message_thread_id (:class:`int`)
            If not 0, a message thread identifier in which the message will be sent
        
        reply_to_message_id (:class:`int`)
            Identifier of a message to reply to or 0
        
        options (:class:`MessageSendOptions`)
            Options to be used to send the message
        
        query_id (:class:`int`)
            Identifier of the inline query
        
        result_id (:class:`str`)
            Identifier of the inline result
        
        hide_via_bot (:class:`bool`)
            If true, there will be no mention of a bot, via which the message is sent. Can be used only for bots GetOption("animation_search_bot_username"), GetOption("photo_search_bot_username") and GetOption("venue_search_bot_username")
        
    """

    ID: str = Field("sendInlineQueryResultMessage", alias="@type")
    chat_id: int
    message_thread_id: int
    reply_to_message_id: int
    options: MessageSendOptions
    query_id: int
    result_id: str
    hide_via_bot: bool

    @staticmethod
    def read(q: dict) -> SendInlineQueryResultMessage:
        return SendInlineQueryResultMessage.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GameHighScore(BaseObject):
    """
    Contains one row of the game high score table
    
    Params:
        position (:class:`int`)
            Position in the high score table
        
        user_id (:class:`int`)
            User identifier
        
        score (:class:`int`)
            User score
        
    """

    ID: str = Field("gameHighScore", alias="@type")
    position: int
    user_id: int
    score: int

    @staticmethod
    def read(q: dict) -> GameHighScore:
        return GameHighScore.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from ..base_object import BaseObject


class StartGroupCallRecording(BaseObject):
    """
    Starts recording of an active group call. Requires groupCall.can_be_managed group call flag
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier
        
        title (:class:`str`)
            Group call recording title; 0-64 characters
        
    """

    ID: str = Field("startGroupCallRecording", alias="@type")
    group_call_id: int
    title: typing.Optional[str] = Field(None, max_length=64)

    @staticmethod
    def read(q: dict) -> StartGroupCallRecording:
        return StartGroupCallRecording.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .update import Update
from ..base_object import BaseObject


class Updates(BaseObject):
    """
    Contains a list of updates
    
    Params:
        updates (:obj:`list[Update]`)
            List of updates
        
    """

    ID: str = Field("updates", alias="@type")
    updates: list[Update]

    @staticmethod
    def read(q: dict) -> Updates:
        return Updates.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from ..base_object import BaseObject
from ..types import ChatLocation


class CreateNewSupergroupChat(BaseObject):
    """
    Creates a new supergroup or channel and sends a corresponding messageSupergroupChatCreate. Returns the newly created chat
    
    Params:
        title (:class:`str`)
            Title of the new chat; 1-128 characters
        
        is_channel (:class:`bool`)
            True, if a channel chat needs to be created
        
        param_description (:class:`str`)
            Chat description; 0-255 characters
        
        location (:class:`ChatLocation`)
            Chat location if a location-based supergroup is being created
        
        for_import (:class:`bool`)
            True, if the supergroup is created for importing messages using importMessage
        
    """

    ID: str = Field("createNewSupergroupChat", alias="@type")
    title: str = Field(..., min_length=1, max_length=128)
    is_channel: bool
    param_description: typing.Optional[str] = Field(None, max_length=255)
    location: ChatLocation
    for_import: bool

    @staticmethod
    def read(q: dict) -> CreateNewSupergroupChat:
        return CreateNewSupergroupChat.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import MessageSender


class JoinGroupCall(BaseObject):
    """
    Joins an active group call. Returns join response payload for tgcalls
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier
        
        participant_id (:class:`MessageSender`)
            Identifier of a group call participant, which will be used to join the call; voice chats only
        
        audio_source_id (:class:`int`)
            Caller audio channel synchronization source identifier; received from tgcalls
        
        payload (:class:`str`)
            Group call join payload; received from tgcalls
        
        is_muted (:class:`bool`)
            True, if the user's microphone is muted
        
        is_my_video_enabled (:class:`bool`)
            True, if the user's video is enabled
        
        invite_hash (:class:`str`)
            If non-empty, invite hash to be used to join the group call without being muted by administrators
        
    """

    ID: str = Field("joinGroupCall", alias="@type")
    group_call_id: int
    participant_id: MessageSender
    audio_source_id: int
    payload: str
    is_muted: bool
    is_my_video_enabled: bool
    invite_hash: str

    @staticmethod
    def read(q: dict) -> JoinGroupCall:
        return JoinGroupCall.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class SearchStickers(BaseObject):
    """
    Searches for stickers from public sticker sets that correspond to a given emoji
    
    Params:
        emoji (:class:`str`)
            String representation of emoji; must be non-empty
        
        limit (:class:`int`)
            The maximum number of stickers to be returned
        
    """

    ID: str = Field("searchStickers", alias="@type")
    emoji: str
    limit: int

    @staticmethod
    def read(q: dict) -> SearchStickers:
        return SearchStickers.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class LocalFile(BaseObject):
    """
    Represents a local file
    
    Params:
        path (:class:`str`)
            Local path to the locally available file part; may be empty
        
        can_be_downloaded (:class:`bool`)
            True, if it is possible to try to download or generate the file
        
        can_be_deleted (:class:`bool`)
            True, if the file can be deleted
        
        is_downloading_active (:class:`bool`)
            True, if the file is currently being downloaded (or a local copy is being generated by some other means)
        
        is_downloading_completed (:class:`bool`)
            True, if the local copy is fully available
        
        download_offset (:class:`int`)
            Download will be started from this offset. downloaded_prefix_size is calculated from this offset
        
        downloaded_prefix_size (:class:`int`)
            If is_downloading_completed is false, then only some prefix of the file starting from download_offset is ready to be read. downloaded_prefix_size is the size of that prefix
        
        downloaded_size (:class:`int`)
            Total downloaded file bytes. Should be used only for calculating download progress. The actual file size may be bigger, and some parts of it may contain garbage
        
    """

    ID: str = Field("localFile", alias="@type")
    path: str
    can_be_downloaded: bool
    can_be_deleted: bool
    is_downloading_active: bool
    is_downloading_completed: bool
    download_offset: int
    downloaded_prefix_size: int
    downloaded_size: int

    @staticmethod
    def read(q: dict) -> LocalFile:
        return LocalFile.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class ViewTrendingStickerSets(BaseObject):
    """
    Informs the server that some trending sticker sets have been viewed by the user
    
    Params:
        sticker_set_ids (:obj:`list[int]`)
            Identifiers of viewed trending sticker sets
        
    """

    ID: str = Field("viewTrendingStickerSets", alias="@type")
    sticker_set_ids: list[int]

    @staticmethod
    def read(q: dict) -> ViewTrendingStickerSets:
        return ViewTrendingStickerSets.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class InlineKeyboardButtonType(BaseObject):
    """
    Describes the type of an inline keyboard button
    
    """

    ID: str = Field("inlineKeyboardButtonType", alias="@type")


class InlineKeyboardButtonTypeBuy(InlineKeyboardButtonType):
    """
    A button to buy something. This button must be in the first column and row of the keyboard and can be attached only to a message with content of the type messageInvoice
    
    """

    ID: str = Field("inlineKeyboardButtonTypeBuy", alias="@type")

    @staticmethod
    def read(q: dict) -> InlineKeyboardButtonTypeBuy:
        return InlineKeyboardButtonTypeBuy.construct(**q)


class InlineKeyboardButtonTypeCallback(InlineKeyboardButtonType):
    """
    A button that sends a callback query to a bot
    
    Params:
        data (:class:`str`)
            Data to be sent to the bot via a callback query
        
    """

    ID: str = Field("inlineKeyboardButtonTypeCallback", alias="@type")
    data: str

    @staticmethod
    def read(q: dict) -> InlineKeyboardButtonTypeCallback:
        return InlineKeyboardButtonTypeCallback.construct(**q)


class InlineKeyboardButtonTypeCallbackGame(InlineKeyboardButtonType):
    """
    A button with a game that sends a callback query to a bot. This button must be in the first column and row of the keyboard and can be attached only to a message with content of the type messageGame
    
    """

    ID: str = Field("inlineKeyboardButtonTypeCallbackGame", alias="@type")

    @staticmethod
    def read(q: dict) -> InlineKeyboardButtonTypeCallbackGame:
        return InlineKeyboardButtonTypeCallbackGame.construct(**q)


class InlineKeyboardButtonTypeCallbackWithPassword(InlineKeyboardButtonType):
    """
    A button that asks for password of the current user and then sends a callback query to a bot
    
    Params:
        data (:class:`str`)
            Data to be sent to the bot via a callback query
        
    """

    ID: str = Field("inlineKeyboardButtonTypeCallbackWithPassword", alias="@type")
    data: str

    @staticmethod
    def read(q: dict) -> InlineKeyboardButtonTypeCallbackWithPassword:
        return InlineKeyboardButtonTypeCallbackWithPassword.construct(**q)


class InlineKeyboardButtonTypeLoginUrl(InlineKeyboardButtonType):
    """
    A button that opens a specified URL and automatically authorize the current user if allowed to do so
    
    Params:
        url (:class:`str`)
            An HTTP URL to open
        
        id (:class:`int`)
            Unique button identifier
        
        forward_text (:class:`str`)
            If non-empty, new text of the button in forwarded messages
        
    """

    ID: str = Field("inlineKeyboardButtonTypeLoginUrl", alias="@type")
    url: str
    id: int
    forward_text: str

    @staticmethod
    def read(q: dict) -> InlineKeyboardButtonTypeLoginUrl:
        return InlineKeyboardButtonTypeLoginUrl.construct(**q)


class InlineKeyboardButtonTypeSwitchInline(InlineKeyboardButtonType):
    """
    A button that forces an inline query to the bot to be inserted in the input field
    
    Params:
        query (:class:`str`)
            Inline query to be sent to the bot
        
        in_current_chat (:class:`bool`)
            True, if the inline query should be sent from the current chat
        
    """

    ID: str = Field("inlineKeyboardButtonTypeSwitchInline", alias="@type")
    query: str
    in_current_chat: bool

    @staticmethod
    def read(q: dict) -> InlineKeyboardButtonTypeSwitchInline:
        return InlineKeyboardButtonTypeSwitchInline.construct(**q)


class InlineKeyboardButtonTypeUrl(InlineKeyboardButtonType):
    """
    A button that opens a specified URL
    
    Params:
        url (:class:`str`)
            HTTP or tg:// URL to open
        
    """

    ID: str = Field("inlineKeyboardButtonTypeUrl", alias="@type")
    url: str

    @staticmethod
    def read(q: dict) -> InlineKeyboardButtonTypeUrl:
        return InlineKeyboardButtonTypeUrl.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class ConfirmQrCodeAuthentication(BaseObject):
    """
    Confirms QR code authentication on another device. Returns created session on success
    
    Params:
        link (:class:`str`)
            A link from a QR code. The link must be scanned by the in-app camera
        
    """

    ID: str = Field("confirmQrCodeAuthentication", alias="@type")
    link: str

    @staticmethod
    def read(q: dict) -> ConfirmQrCodeAuthentication:
        return ConfirmQrCodeAuthentication.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .sticker_set_info import StickerSetInfo
from ..base_object import BaseObject


class StickerSets(BaseObject):
    """
    Represents a list of sticker sets
    
    Params:
        total_count (:class:`int`)
            Approximate total number of sticker sets found
        
        sets (:obj:`list[StickerSetInfo]`)
            List of sticker sets
        
    """

    ID: str = Field("stickerSets", alias="@type")
    total_count: int
    sets: list[StickerSetInfo]

    @staticmethod
    def read(q: dict) -> StickerSets:
        return StickerSets.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import InputMessageContent
from ..types import MessageSender


class AddLocalMessage(BaseObject):
    """
    Adds a local message to a chat. The message is persistent across application restarts only if the message database is used. Returns the added message
    
    Params:
        chat_id (:class:`int`)
            Target chat
        
        sender (:class:`MessageSender`)
            The sender sender of the message
        
        reply_to_message_id (:class:`int`)
            Identifier of the message to reply to or 0
        
        disable_notification (:class:`bool`)
            Pass true to disable notification for the message
        
        input_message_content (:class:`InputMessageContent`)
            The content of the message to be added
        
    """

    ID: str = Field("addLocalMessage", alias="@type")
    chat_id: int
    sender: MessageSender
    reply_to_message_id: int
    disable_notification: bool
    input_message_content: InputMessageContent

    @staticmethod
    def read(q: dict) -> AddLocalMessage:
        return AddLocalMessage.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class CheckRecoveryEmailAddressCode(BaseObject):
    """
    Checks the 2-step verification recovery email address verification code
    
    Params:
        code (:class:`str`)
            Verification code
        
    """

    ID: str = Field("checkRecoveryEmailAddressCode", alias="@type")
    code: str

    @staticmethod
    def read(q: dict) -> CheckRecoveryEmailAddressCode:
        return CheckRecoveryEmailAddressCode.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class CheckAuthenticationPasswordRecoveryCode(BaseObject):
    """
    Checks whether a password recovery code sent to an email address is valid. Works only when the current authorization state is authorizationStateWaitPassword
    
    Params:
        recovery_code (:class:`str`)
            Recovery code to check
        
    """

    ID: str = Field("checkAuthenticationPasswordRecoveryCode", alias="@type")
    recovery_code: str

    @staticmethod
    def read(q: dict) -> CheckAuthenticationPasswordRecoveryCode:
        return CheckAuthenticationPasswordRecoveryCode.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import FormattedText


class GetMarkdownText(BaseObject):
    """
    Replaces text entities with Markdown formatting in a human-friendly format. Entities that can't be represented in Markdown unambiguously are kept as is. Can be called synchronously
    
    Params:
        text (:class:`FormattedText`)
            The text
        
    """

    ID: str = Field("getMarkdownText", alias="@type")
    text: FormattedText

    @staticmethod
    def read(q: dict) -> GetMarkdownText:
        return GetMarkdownText.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .mask_point import MaskPoint
from ..base_object import BaseObject


class MaskPosition(BaseObject):
    """
    Position on a photo where a mask should be placed
    
    Params:
        point (:class:`MaskPoint`)
            Part of the face, relative to which the mask should be placed
        
        x_shift (:class:`float`)
            Shift by X-axis measured in widths of the mask scaled to the face size, from left to right. (For example, -1.0 will place the mask just to the left of the default mask position)
        
        y_shift (:class:`float`)
            Shift by Y-axis measured in heights of the mask scaled to the face size, from top to bottom. (For example, 1.0 will place the mask just below the default mask position)
        
        scale (:class:`float`)
            Mask scaling coefficient. (For example, 2.0 means a doubled size)
        
    """

    ID: str = Field("maskPosition", alias="@type")
    point: MaskPoint
    x_shift: float
    y_shift: float
    scale: float

    @staticmethod
    def read(q: dict) -> MaskPosition:
        return MaskPosition.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetPollVoters(BaseObject):
    """
    Returns users voted for the specified option in a non-anonymous polls. For the optimal performance the number of returned users is chosen by the library
    
    Params:
        chat_id (:class:`int`)
            Identifier of the chat to which the poll belongs
        
        message_id (:class:`int`)
            Identifier of the message containing the poll
        
        option_id (:class:`int`)
            0-based identifier of the answer option
        
        offset (:class:`int`)
            Number of users to skip in the result; must be non-negative
        
        limit (:class:`int`)
            The maximum number of users to be returned; must be positive and can't be greater than 50. Fewer users may be returned than specified by the limit, even if the end of the voter list has not been reached
        
    """

    ID: str = Field("getPollVoters", alias="@type")
    chat_id: int
    message_id: int
    option_id: int
    offset: int
    limit: int

    @staticmethod
    def read(q: dict) -> GetPollVoters:
        return GetPollVoters.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class PassportElementErrorSource(BaseObject):
    """
    Contains the description of an error in a Telegram Passport element
    
    """

    ID: str = Field("passportElementErrorSource", alias="@type")


class PassportElementErrorSourceDataField(PassportElementErrorSource):
    """
    One of the data fields contains an error. The error will be considered resolved when the value of the field changes
    
    Params:
        field_name (:class:`str`)
            Field name
        
    """

    ID: str = Field("passportElementErrorSourceDataField", alias="@type")
    field_name: str

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceDataField:
        return PassportElementErrorSourceDataField.construct(**q)


class PassportElementErrorSourceFile(PassportElementErrorSource):
    """
    The file contains an error. The error will be considered resolved when the file changes
    
    Params:
        file_index (:class:`int`)
            Index of a file with the error
        
    """

    ID: str = Field("passportElementErrorSourceFile", alias="@type")
    file_index: int

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceFile:
        return PassportElementErrorSourceFile.construct(**q)


class PassportElementErrorSourceFiles(PassportElementErrorSource):
    """
    The list of attached files contains an error. The error will be considered resolved when the list of files changes
    
    """

    ID: str = Field("passportElementErrorSourceFiles", alias="@type")

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceFiles:
        return PassportElementErrorSourceFiles.construct(**q)


class PassportElementErrorSourceFrontSide(PassportElementErrorSource):
    """
    The front side of the document contains an error. The error will be considered resolved when the file with the front side changes
    
    """

    ID: str = Field("passportElementErrorSourceFrontSide", alias="@type")

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceFrontSide:
        return PassportElementErrorSourceFrontSide.construct(**q)


class PassportElementErrorSourceReverseSide(PassportElementErrorSource):
    """
    The reverse side of the document contains an error. The error will be considered resolved when the file with the reverse side changes
    
    """

    ID: str = Field("passportElementErrorSourceReverseSide", alias="@type")

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceReverseSide:
        return PassportElementErrorSourceReverseSide.construct(**q)


class PassportElementErrorSourceSelfie(PassportElementErrorSource):
    """
    The selfie with the document contains an error. The error will be considered resolved when the file with the selfie changes
    
    """

    ID: str = Field("passportElementErrorSourceSelfie", alias="@type")

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceSelfie:
        return PassportElementErrorSourceSelfie.construct(**q)


class PassportElementErrorSourceTranslationFile(PassportElementErrorSource):
    """
    One of files with the translation of the document contains an error. The error will be considered resolved when the file changes
    
    Params:
        file_index (:class:`int`)
            Index of a file with the error
        
    """

    ID: str = Field("passportElementErrorSourceTranslationFile", alias="@type")
    file_index: int

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceTranslationFile:
        return PassportElementErrorSourceTranslationFile.construct(**q)


class PassportElementErrorSourceTranslationFiles(PassportElementErrorSource):
    """
    The translation of the document contains an error. The error will be considered resolved when the list of translation files changes
    
    """

    ID: str = Field("passportElementErrorSourceTranslationFiles", alias="@type")

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceTranslationFiles:
        return PassportElementErrorSourceTranslationFiles.construct(**q)


class PassportElementErrorSourceUnspecified(PassportElementErrorSource):
    """
    The element contains an error in an unspecified place. The error will be considered resolved when new data is added
    
    """

    ID: str = Field("passportElementErrorSourceUnspecified", alias="@type")

    @staticmethod
    def read(q: dict) -> PassportElementErrorSourceUnspecified:
        return PassportElementErrorSourceUnspecified.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .animation import Animation
from .audio import Audio
from .document import Document
from .photo import Photo
from .sticker import Sticker
from .video import Video
from .video_note import VideoNote
from .voice_note import VoiceNote
from ..base_object import BaseObject


class PushMessageContent(BaseObject):
    """
    Contains content of a push message notification
    
    """

    ID: str = Field("pushMessageContent", alias="@type")


class PushMessageContentAnimation(PushMessageContent):
    """
    An animation message (GIF-style).
    
    Params:
        animation (:class:`Animation`)
            Message content; may be null
        
        caption (:class:`str`)
            Animation caption
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentAnimation", alias="@type")
    animation: typing.Optional[Animation] = None
    caption: str
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentAnimation:
        return PushMessageContentAnimation.construct(**q)


class PushMessageContentAudio(PushMessageContent):
    """
    An audio message
    
    Params:
        audio (:class:`Audio`)
            Message content; may be null
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentAudio", alias="@type")
    audio: typing.Optional[Audio] = None
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentAudio:
        return PushMessageContentAudio.construct(**q)


class PushMessageContentBasicGroupChatCreate(PushMessageContent):
    """
    A newly created basic group
    
    """

    ID: str = Field("pushMessageContentBasicGroupChatCreate", alias="@type")

    @staticmethod
    def read(q: dict) -> PushMessageContentBasicGroupChatCreate:
        return PushMessageContentBasicGroupChatCreate.construct(**q)


class PushMessageContentChatAddMembers(PushMessageContent):
    """
    New chat members were invited to a group
    
    Params:
        member_name (:class:`str`)
            Name of the added member
        
        is_current_user (:class:`bool`)
            True, if the current user was added to the group
        
        is_returned (:class:`bool`)
            True, if the user has returned to the group themselves
        
    """

    ID: str = Field("pushMessageContentChatAddMembers", alias="@type")
    member_name: str
    is_current_user: bool
    is_returned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentChatAddMembers:
        return PushMessageContentChatAddMembers.construct(**q)


class PushMessageContentChatChangePhoto(PushMessageContent):
    """
    A chat photo was edited
    
    """

    ID: str = Field("pushMessageContentChatChangePhoto", alias="@type")

    @staticmethod
    def read(q: dict) -> PushMessageContentChatChangePhoto:
        return PushMessageContentChatChangePhoto.construct(**q)


class PushMessageContentChatChangeTitle(PushMessageContent):
    """
    A chat title was edited
    
    Params:
        title (:class:`str`)
            New chat title
        
    """

    ID: str = Field("pushMessageContentChatChangeTitle", alias="@type")
    title: str

    @staticmethod
    def read(q: dict) -> PushMessageContentChatChangeTitle:
        return PushMessageContentChatChangeTitle.construct(**q)


class PushMessageContentChatDeleteMember(PushMessageContent):
    """
    A chat member was deleted
    
    Params:
        member_name (:class:`str`)
            Name of the deleted member
        
        is_current_user (:class:`bool`)
            True, if the current user was deleted from the group
        
        is_left (:class:`bool`)
            True, if the user has left the group themselves
        
    """

    ID: str = Field("pushMessageContentChatDeleteMember", alias="@type")
    member_name: str
    is_current_user: bool
    is_left: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentChatDeleteMember:
        return PushMessageContentChatDeleteMember.construct(**q)


class PushMessageContentChatJoinByLink(PushMessageContent):
    """
    A new member joined the chat by invite link
    
    """

    ID: str = Field("pushMessageContentChatJoinByLink", alias="@type")

    @staticmethod
    def read(q: dict) -> PushMessageContentChatJoinByLink:
        return PushMessageContentChatJoinByLink.construct(**q)


class PushMessageContentContact(PushMessageContent):
    """
    A message with a user contact
    
    Params:
        name (:class:`str`)
            Contact's name
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentContact", alias="@type")
    name: str
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentContact:
        return PushMessageContentContact.construct(**q)


class PushMessageContentContactRegistered(PushMessageContent):
    """
    A contact has registered with Telegram
    
    """

    ID: str = Field("pushMessageContentContactRegistered", alias="@type")

    @staticmethod
    def read(q: dict) -> PushMessageContentContactRegistered:
        return PushMessageContentContactRegistered.construct(**q)


class PushMessageContentDocument(PushMessageContent):
    """
    A document message (a general file)
    
    Params:
        document (:class:`Document`)
            Message content; may be null
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentDocument", alias="@type")
    document: typing.Optional[Document] = None
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentDocument:
        return PushMessageContentDocument.construct(**q)


class PushMessageContentGame(PushMessageContent):
    """
    A message with a game
    
    Params:
        title (:class:`str`)
            Game title, empty for pinned game message
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentGame", alias="@type")
    title: str
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentGame:
        return PushMessageContentGame.construct(**q)


class PushMessageContentGameScore(PushMessageContent):
    """
    A new high score was achieved in a game
    
    Params:
        title (:class:`str`)
            Game title, empty for pinned message
        
        score (:class:`int`)
            New score, 0 for pinned message
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentGameScore", alias="@type")
    title: str
    score: int
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentGameScore:
        return PushMessageContentGameScore.construct(**q)


class PushMessageContentHidden(PushMessageContent):
    """
    A general message with hidden content
    
    Params:
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentHidden", alias="@type")
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentHidden:
        return PushMessageContentHidden.construct(**q)


class PushMessageContentInvoice(PushMessageContent):
    """
    A message with an invoice from a bot
    
    Params:
        price (:class:`str`)
            Product price
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentInvoice", alias="@type")
    price: str
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentInvoice:
        return PushMessageContentInvoice.construct(**q)


class PushMessageContentLocation(PushMessageContent):
    """
    A message with a location
    
    Params:
        is_live (:class:`bool`)
            True, if the location is live
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentLocation", alias="@type")
    is_live: bool
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentLocation:
        return PushMessageContentLocation.construct(**q)


class PushMessageContentMediaAlbum(PushMessageContent):
    """
    A media album
    
    Params:
        total_count (:class:`int`)
            Number of messages in the album
        
        has_photos (:class:`bool`)
            True, if the album has at least one photo
        
        has_videos (:class:`bool`)
            True, if the album has at least one video
        
        has_audios (:class:`bool`)
            True, if the album has at least one audio file
        
        has_documents (:class:`bool`)
            True, if the album has at least one document
        
    """

    ID: str = Field("pushMessageContentMediaAlbum", alias="@type")
    total_count: int
    has_photos: bool
    has_videos: bool
    has_audios: bool
    has_documents: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentMediaAlbum:
        return PushMessageContentMediaAlbum.construct(**q)


class PushMessageContentMessageForwards(PushMessageContent):
    """
    A forwarded messages
    
    Params:
        total_count (:class:`int`)
            Number of forwarded messages
        
    """

    ID: str = Field("pushMessageContentMessageForwards", alias="@type")
    total_count: int

    @staticmethod
    def read(q: dict) -> PushMessageContentMessageForwards:
        return PushMessageContentMessageForwards.construct(**q)


class PushMessageContentPhoto(PushMessageContent):
    """
    A photo message
    
    Params:
        photo (:class:`Photo`)
            Message content; may be null
        
        caption (:class:`str`)
            Photo caption
        
        is_secret (:class:`bool`)
            True, if the photo is secret
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentPhoto", alias="@type")
    photo: typing.Optional[Photo] = None
    caption: str
    is_secret: bool
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentPhoto:
        return PushMessageContentPhoto.construct(**q)


class PushMessageContentPoll(PushMessageContent):
    """
    A message with a poll
    
    Params:
        question (:class:`str`)
            Poll question
        
        is_regular (:class:`bool`)
            True, if the poll is regular and not in quiz mode
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentPoll", alias="@type")
    question: str
    is_regular: bool
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentPoll:
        return PushMessageContentPoll.construct(**q)


class PushMessageContentScreenshotTaken(PushMessageContent):
    """
    A screenshot of a message in the chat has been taken
    
    """

    ID: str = Field("pushMessageContentScreenshotTaken", alias="@type")

    @staticmethod
    def read(q: dict) -> PushMessageContentScreenshotTaken:
        return PushMessageContentScreenshotTaken.construct(**q)


class PushMessageContentSticker(PushMessageContent):
    """
    A message with a sticker
    
    Params:
        sticker (:class:`Sticker`)
            Message content; may be null
        
        emoji (:class:`str`)
            Emoji corresponding to the sticker; may be empty
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentSticker", alias="@type")
    sticker: typing.Optional[Sticker] = None
    emoji: str
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentSticker:
        return PushMessageContentSticker.construct(**q)


class PushMessageContentText(PushMessageContent):
    """
    A text message
    
    Params:
        text (:class:`str`)
            Message text
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentText", alias="@type")
    text: str
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentText:
        return PushMessageContentText.construct(**q)


class PushMessageContentVideo(PushMessageContent):
    """
    A video message
    
    Params:
        video (:class:`Video`)
            Message content; may be null
        
        caption (:class:`str`)
            Video caption
        
        is_secret (:class:`bool`)
            True, if the video is secret
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentVideo", alias="@type")
    video: typing.Optional[Video] = None
    caption: str
    is_secret: bool
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentVideo:
        return PushMessageContentVideo.construct(**q)


class PushMessageContentVideoNote(PushMessageContent):
    """
    A video note message
    
    Params:
        video_note (:class:`VideoNote`)
            Message content; may be null
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentVideoNote", alias="@type")
    video_note: typing.Optional[VideoNote] = None
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentVideoNote:
        return PushMessageContentVideoNote.construct(**q)


class PushMessageContentVoiceNote(PushMessageContent):
    """
    A voice note message
    
    Params:
        voice_note (:class:`VoiceNote`)
            Message content; may be null
        
        is_pinned (:class:`bool`)
            True, if the message is a pinned message with the specified content
        
    """

    ID: str = Field("pushMessageContentVoiceNote", alias="@type")
    voice_note: typing.Optional[VoiceNote] = None
    is_pinned: bool

    @staticmethod
    def read(q: dict) -> PushMessageContentVoiceNote:
        return PushMessageContentVoiceNote.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject
from ..types import ChatPermissions


class SetChatPermissions(BaseObject):
    """
    Changes the chat members permissions. Supported only for basic groups and supergroups. Requires can_restrict_members administrator right
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        permissions (:class:`ChatPermissions`)
            New non-administrator members permissions in the chat
        
    """

    ID: str = Field("setChatPermissions", alias="@type")
    chat_id: int
    permissions: ChatPermissions

    @staticmethod
    def read(q: dict) -> SetChatPermissions:
        return SetChatPermissions.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class StartGroupCallScreenSharing(BaseObject):
    """
    Starts screen sharing in a joined group call. Returns join response payload for tgcalls
    
    Params:
        group_call_id (:class:`int`)
            Group call identifier
        
        audio_source_id (:class:`int`)
            Screen sharing audio channel synchronization source identifier; received from tgcalls
        
        payload (:class:`str`)
            Group call join payload; received from tgcalls
        
    """

    ID: str = Field("startGroupCallScreenSharing", alias="@type")
    group_call_id: int
    audio_source_id: int
    payload: str

    @staticmethod
    def read(q: dict) -> StartGroupCallScreenSharing:
        return StartGroupCallScreenSharing.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .document import Document
from ..base_object import BaseObject


class RichText(BaseObject):
    """
    Describes a text object inside an instant-view web page
    
    """

    ID: str = Field("richText", alias="@type")


class RichTextAnchor(RichText):
    """
    An anchor
    
    Params:
        name (:class:`str`)
            Anchor name
        
    """

    ID: str = Field("richTextAnchor", alias="@type")
    name: str

    @staticmethod
    def read(q: dict) -> RichTextAnchor:
        return RichTextAnchor.construct(**q)


class RichTextAnchorLink(RichText):
    """
    A link to an anchor on the same web page
    
    Params:
        text (:class:`RichText`)
            The link text
        
        anchor_name (:class:`str`)
            The anchor name. If the name is empty, the link should bring back to top
        
        url (:class:`str`)
            An HTTP URL, opening the anchor
        
    """

    ID: str = Field("richTextAnchorLink", alias="@type")
    text: RichText
    anchor_name: str
    url: str

    @staticmethod
    def read(q: dict) -> RichTextAnchorLink:
        return RichTextAnchorLink.construct(**q)


class RichTextBold(RichText):
    """
    A bold rich text
    
    Params:
        text (:class:`RichText`)
            Text
        
    """

    ID: str = Field("richTextBold", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> RichTextBold:
        return RichTextBold.construct(**q)


class RichTextEmailAddress(RichText):
    """
    A rich text email link
    
    Params:
        text (:class:`RichText`)
            Text
        
        email_address (:class:`str`)
            Email address
        
    """

    ID: str = Field("richTextEmailAddress", alias="@type")
    text: RichText
    email_address: str

    @staticmethod
    def read(q: dict) -> RichTextEmailAddress:
        return RichTextEmailAddress.construct(**q)


class RichTextFixed(RichText):
    """
    A fixed-width rich text
    
    Params:
        text (:class:`RichText`)
            Text
        
    """

    ID: str = Field("richTextFixed", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> RichTextFixed:
        return RichTextFixed.construct(**q)


class RichTextIcon(RichText):
    """
    A small image inside the text
    
    Params:
        document (:class:`Document`)
            The image represented as a document. The image can be in GIF, JPEG or PNG format
        
        width (:class:`int`)
            Width of a bounding box in which the image should be shown; 0 if unknown
        
        height (:class:`int`)
            Height of a bounding box in which the image should be shown; 0 if unknown
        
    """

    ID: str = Field("richTextIcon", alias="@type")
    document: Document
    width: int
    height: int

    @staticmethod
    def read(q: dict) -> RichTextIcon:
        return RichTextIcon.construct(**q)


class RichTextItalic(RichText):
    """
    An italicized rich text
    
    Params:
        text (:class:`RichText`)
            Text
        
    """

    ID: str = Field("richTextItalic", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> RichTextItalic:
        return RichTextItalic.construct(**q)


class RichTextMarked(RichText):
    """
    A marked rich text
    
    Params:
        text (:class:`RichText`)
            Text
        
    """

    ID: str = Field("richTextMarked", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> RichTextMarked:
        return RichTextMarked.construct(**q)


class RichTextPhoneNumber(RichText):
    """
    A rich text phone number
    
    Params:
        text (:class:`RichText`)
            Text
        
        phone_number (:class:`str`)
            Phone number
        
    """

    ID: str = Field("richTextPhoneNumber", alias="@type")
    text: RichText
    phone_number: str

    @staticmethod
    def read(q: dict) -> RichTextPhoneNumber:
        return RichTextPhoneNumber.construct(**q)


class RichTextPlain(RichText):
    """
    A plain text
    
    Params:
        text (:class:`str`)
            Text
        
    """

    ID: str = Field("richTextPlain", alias="@type")
    text: str

    @staticmethod
    def read(q: dict) -> RichTextPlain:
        return RichTextPlain.construct(**q)


class RichTextReference(RichText):
    """
    A reference to a richTexts object on the same web page
    
    Params:
        text (:class:`RichText`)
            The text
        
        anchor_name (:class:`str`)
            The name of a richTextAnchor object, which is the first element of the target richTexts object
        
        url (:class:`str`)
            An HTTP URL, opening the reference
        
    """

    ID: str = Field("richTextReference", alias="@type")
    text: RichText
    anchor_name: str
    url: str

    @staticmethod
    def read(q: dict) -> RichTextReference:
        return RichTextReference.construct(**q)


class RichTextStrikethrough(RichText):
    """
    A strikethrough rich text
    
    Params:
        text (:class:`RichText`)
            Text
        
    """

    ID: str = Field("richTextStrikethrough", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> RichTextStrikethrough:
        return RichTextStrikethrough.construct(**q)


class RichTextSubscript(RichText):
    """
    A subscript rich text
    
    Params:
        text (:class:`RichText`)
            Text
        
    """

    ID: str = Field("richTextSubscript", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> RichTextSubscript:
        return RichTextSubscript.construct(**q)


class RichTextSuperscript(RichText):
    """
    A superscript rich text
    
    Params:
        text (:class:`RichText`)
            Text
        
    """

    ID: str = Field("richTextSuperscript", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> RichTextSuperscript:
        return RichTextSuperscript.construct(**q)


class RichTextUnderline(RichText):
    """
    An underlined rich text
    
    Params:
        text (:class:`RichText`)
            Text
        
    """

    ID: str = Field("richTextUnderline", alias="@type")
    text: RichText

    @staticmethod
    def read(q: dict) -> RichTextUnderline:
        return RichTextUnderline.construct(**q)


class RichTextUrl(RichText):
    """
    A rich text URL link
    
    Params:
        text (:class:`RichText`)
            Text
        
        url (:class:`str`)
            URL
        
        is_cached (:class:`bool`)
            True, if the URL has cached instant view server-side
        
    """

    ID: str = Field("richTextUrl", alias="@type")
    text: RichText
    url: str
    is_cached: bool

    @staticmethod
    def read(q: dict) -> RichTextUrl:
        return RichTextUrl.construct(**q)


class RichTexts(RichText):
    """
    A concatenation of rich texts
    
    Params:
        texts (:obj:`list[RichText]`)
            Texts
        
    """

    ID: str = Field("richTexts", alias="@type")
    texts: list[RichText]

    @staticmethod
    def read(q: dict) -> RichTexts:
        return RichTexts.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

import typing

from pydantic import Field

from .closed_vector_path import ClosedVectorPath
from .file import File
from .mask_position import MaskPosition
from .thumbnail import Thumbnail
from ..base_object import BaseObject


class Sticker(BaseObject):
    """
    Describes a sticker
    
    Params:
        set_id (:class:`int`)
            The identifier of the sticker set to which the sticker belongs; 0 if none
        
        width (:class:`int`)
            Sticker width; as defined by the sender
        
        height (:class:`int`)
            Sticker height; as defined by the sender
        
        emoji (:class:`str`)
            Emoji corresponding to the sticker
        
        is_animated (:class:`bool`)
            True, if the sticker is an animated sticker in TGS format
        
        is_mask (:class:`bool`)
            True, if the sticker is a mask
        
        mask_position (:class:`MaskPosition`)
            Position where the mask should be placed; may be null
        
        outline (:obj:`list[ClosedVectorPath]`)
            Sticker's outline represented as a list of closed vector paths; may be empty. The coordinate system origin is in the upper-left corner
        
        thumbnail (:class:`Thumbnail`)
            Sticker thumbnail in WEBP or JPEG format; may be null
        
        sticker (:class:`File`)
            File containing the sticker
        
    """

    ID: str = Field("sticker", alias="@type")
    set_id: int
    width: int
    height: int
    emoji: str
    is_animated: bool
    is_mask: bool
    mask_position: typing.Optional[MaskPosition] = None
    outline: list[ClosedVectorPath]
    thumbnail: typing.Optional[Thumbnail] = None
    sticker: File

    @staticmethod
    def read(q: dict) -> Sticker:
        return Sticker.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class GetMessagePublicForwards(BaseObject):
    """
    Returns forwarded copies of a channel message to different public channels. For optimal performance the number of returned messages is chosen by the library
    
    Params:
        chat_id (:class:`int`)
            Chat identifier of the message
        
        message_id (:class:`int`)
            Message identifier
        
        offset (:class:`str`)
            Offset of the first entry to return as received from the previous request; use empty string to get first chunk of results
        
        limit (:class:`int`)
            The maximum number of messages to be returned; must be positive and can't be greater than 100. Fewer messages may be returned than specified by the limit, even if the end of the list has not been reached
        
    """

    ID: str = Field("getMessagePublicForwards", alias="@type")
    chat_id: int
    message_id: int
    offset: str
    limit: int

    @staticmethod
    def read(q: dict) -> GetMessagePublicForwards:
        return GetMessagePublicForwards.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class ChangeStickerSet(BaseObject):
    """
    Installs/uninstalls or activates/archives a sticker set
    
    Params:
        set_id (:class:`int`)
            Identifier of the sticker set
        
        is_installed (:class:`bool`)
            The new value of is_installed
        
        is_archived (:class:`bool`)
            The new value of is_archived. A sticker set can't be installed and archived simultaneously
        
    """

    ID: str = Field("changeStickerSet", alias="@type")
    set_id: int
    is_installed: bool
    is_archived: bool

    @staticmethod
    def read(q: dict) -> ChangeStickerSet:
        return ChangeStickerSet.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from ..base_object import BaseObject


class BotCommandScope(BaseObject):
    """
    Represents the scope to which bot commands are relevant
    
    """

    ID: str = Field("botCommandScope", alias="@type")


class BotCommandScopeAllChatAdministrators(BotCommandScope):
    """
    A scope covering all group and supergroup chat administrators
    
    """

    ID: str = Field("botCommandScopeAllChatAdministrators", alias="@type")

    @staticmethod
    def read(q: dict) -> BotCommandScopeAllChatAdministrators:
        return BotCommandScopeAllChatAdministrators.construct(**q)


class BotCommandScopeAllGroupChats(BotCommandScope):
    """
    A scope covering all group and supergroup chats
    
    """

    ID: str = Field("botCommandScopeAllGroupChats", alias="@type")

    @staticmethod
    def read(q: dict) -> BotCommandScopeAllGroupChats:
        return BotCommandScopeAllGroupChats.construct(**q)


class BotCommandScopeAllPrivateChats(BotCommandScope):
    """
    A scope covering all private chats
    
    """

    ID: str = Field("botCommandScopeAllPrivateChats", alias="@type")

    @staticmethod
    def read(q: dict) -> BotCommandScopeAllPrivateChats:
        return BotCommandScopeAllPrivateChats.construct(**q)


class BotCommandScopeChat(BotCommandScope):
    """
    A scope covering all members of a chat
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
    """

    ID: str = Field("botCommandScopeChat", alias="@type")
    chat_id: int

    @staticmethod
    def read(q: dict) -> BotCommandScopeChat:
        return BotCommandScopeChat.construct(**q)


class BotCommandScopeChatAdministrators(BotCommandScope):
    """
    A scope covering all administrators of a chat
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
    """

    ID: str = Field("botCommandScopeChatAdministrators", alias="@type")
    chat_id: int

    @staticmethod
    def read(q: dict) -> BotCommandScopeChatAdministrators:
        return BotCommandScopeChatAdministrators.construct(**q)


class BotCommandScopeChatMember(BotCommandScope):
    """
    A scope covering a member of a chat
    
    Params:
        chat_id (:class:`int`)
            Chat identifier
        
        user_id (:class:`int`)
            User identifier
        
    """

    ID: str = Field("botCommandScopeChatMember", alias="@type")
    chat_id: int
    user_id: int

    @staticmethod
    def read(q: dict) -> BotCommandScopeChatMember:
        return BotCommandScopeChatMember.construct(**q)


class BotCommandScopeDefault(BotCommandScope):
    """
    A scope covering all users
    
    """

    ID: str = Field("botCommandScopeDefault", alias="@type")

    @staticmethod
    def read(q: dict) -> BotCommandScopeDefault:
        return BotCommandScopeDefault.construct(**q)



# =============================================================================== #
#                                                                                 #
#    This file has been generated automatically!! Do not change this manually!    #
#                                                                                 #
# =============================================================================== #
from __future__ import annotations

from pydantic import Field

from .formatted_text import FormattedText
from ..base_object import BaseObject


class MessageCopyOptions(BaseObject):
    """
    Options to be used when a message content is copied without a link to the original message. Service messages and messageInvoice can't be copied
    
    Params:
        send_copy (:class:`bool`)
            True, if content of the message needs to be copied without a link to the original message. Always true if the message is forwarded to a secret chat
        
        replace_caption (:class:`bool`)
            True, if media caption of the message copy needs to be replaced. Ignored if send_copy is false
        
        new_caption (:class:`FormattedText`)
            New message caption. Ignored if replace_caption is false
        
    """

    ID: str = Field("messageCopyOptions", alias="@type")
    send_copy: bool
    replace_caption: bool
    new_caption: FormattedText

    @staticmethod
    def read(q: dict) -> MessageCopyOptions:
        return MessageCopyOptions.construct(**q)



import asyncio
import base64
import getpass
import logging
import sys
from functools import partial
from typing import (
    Optional,
    TYPE_CHECKING,
    Union,
)

from .api import (
    BaseObject,
    Error,
)
from .api.errors import (
    AioTDLibError,
    Unauthorized,
)

if TYPE_CHECKING:
    from .client import Client

logger = logging.getLogger(__name__)


async def ainput(prompt: str = "", secured: bool = False) -> str:
    if len(prompt):
        prompt = prompt.strip() + " "

    if secured:
        getpass_func = partial(getpass.getpass, prompt=prompt)
        stdin = await asyncio.get_event_loop().run_in_executor(None, getpass_func)
    else:
        print(prompt, end=" ", flush=True)
        stdin = await asyncio.get_event_loop().run_in_executor(None, sys.stdin.readline)

    return stdin.strip()


def str_to_base64(text: Union[str, bytes]) -> str:
    if not text:
        return ''

    result = text

    if isinstance(result, str):
        result = result.encode()

    return base64.b64encode(result).decode()


class PendingRequest:
    request: Optional[BaseObject] = None
    update: Optional[BaseObject] = None
    error: bool = False

    def __init__(self, client: 'Client', request: BaseObject) -> None:
        self.client = client
        self.request = request
        self.__ready_event = asyncio.Event()

    @property
    def id(self) -> Optional[str]:
        return self.request.EXTRA.get('request_id')

    async def wait(self, timeout: Union[int, float] = None, raise_exc: bool = False) -> None:
        result = await asyncio.wait_for(self.__ready_event.wait(), timeout=timeout)

        if result is False:
            raise asyncio.TimeoutError()

        if raise_exc and self.error:
            self.raise_error()

    def set_update(self, update: BaseObject):
        if isinstance(update, Error):
            self.error = True

        self.update = update
        self.__ready_event.set()

    def raise_error(self):
        if isinstance(self.update, Error):
            if self.update.code == 401:
                raise Unauthorized(self.update.message)

            raise AioTDLibError(self.update.code, self.update.message)

        raise RuntimeError(f'Unknown TDLib error')
